@@ -41,6 +41,11 @@
 #include "belle-sip/http-listener.h"
 #include "belle-sip/http-message.h"
 #include "belle-sip/belle-sdp.h"
+#include "belle-sip/bodyhandler.h"
+
+#ifdef ANDROID
+#include "belle-sip/wakelock.h"
+#endif
 
 
 #define BELLE_SIP_POINTER_TO_INT(p)	((int)(long)(p))
