@@ -21,6 +21,7 @@
 
 #include "belle-sip/defs.h"
 #include "belle-sip/sip-uri.h"
+#include "belle-sip/generic-uri.h"
 #include "belle-sip/utils.h"
 #include "belle-sip/parameters.h"
 
@@ -46,17 +47,36 @@ BELLESIP_EXPORT belle_sip_header_address_t* belle_sip_header_address_new();
  * */
 BELLESIP_EXPORT belle_sip_header_address_t* belle_sip_header_address_create(const char* display, belle_sip_uri_t* uri);
 
+/*
+ * creates an address from a display name and an absolute uri
+ * Note the uri not copied but only its ref count is incremented
+ * @param  display display name. May be null.
+ * @param uri uri set to the newly created header_address
+ * @return
+ * */
+BELLESIP_EXPORT belle_sip_header_address_t* belle_sip_header_address_create2(const char* display, belle_generic_uri_t* uri);
+
+
 BELLESIP_EXPORT belle_sip_header_address_t* belle_sip_header_address_parse (const char* address) ;
 
 /**
- *
+ * returns a sip uri. A header address cannot have both a sip uri and an absolute uri.
  */
 BELLESIP_EXPORT belle_sip_uri_t* belle_sip_header_address_get_uri(const belle_sip_header_address_t* address);
 /**
- *
+ * set an absolute uri. A header address cannot have both a sip uri and an absolute uri. This function also to absolute uri to NULL
  */
 BELLESIP_EXPORT void belle_sip_header_address_set_uri(belle_sip_header_address_t* address, belle_sip_uri_t* uri);
 
+/**
+ * returns an absolute uri. A header address cannot have both a sip uri and an absolute uri.
+ */
+BELLESIP_EXPORT belle_generic_uri_t* belle_sip_header_address_get_absolute_uri(const belle_sip_header_address_t* address);
+/**
+ * set an absolute uri. A header address cannot have both a sip uri and an absolute uri. This function also to uri to NULL
+ */
+BELLESIP_EXPORT void belle_sip_header_address_set_absolute_uri(belle_sip_header_address_t* address, belle_generic_uri_t* uri);
+
 /**
  *
  */
@@ -650,6 +670,22 @@ BELLESIP_EXPORT belle_sip_list_t* belle_sip_header_privacy_get_privacy(const bel
 #define BELLE_SIP_HEADER_PRIVACY(t) BELLE_SIP_CAST(t,belle_sip_header_privacy_t)
 #define BELLE_SIP_PRIVACY "Privacy"
 
+
+/******************************
+* Event header object inherent from parameters
+*
+******************************/
+typedef struct _belle_sip_header_event belle_sip_header_event_t;
+BELLESIP_EXPORT belle_sip_header_event_t* belle_sip_header_event_new();
+BELLESIP_EXPORT belle_sip_header_event_t* belle_sip_header_event_parse(const char* event) ;
+BELLESIP_EXPORT belle_sip_header_event_t* belle_sip_header_event_create(const char* event);
+BELLESIP_EXPORT const char* belle_sip_header_event_get_package_name(const belle_sip_header_event_t* event);
+BELLESIP_EXPORT void belle_sip_header_event_set_package_name(belle_sip_header_event_t* event, const char* package_name);
+BELLESIP_EXPORT const char* belle_sip_header_event_get_id(const belle_sip_header_event_t* event);
+BELLESIP_EXPORT void belle_sip_header_event_set_id(belle_sip_header_event_t* event, const char* id);
+#define BELLE_SIP_HEADER_EVENT(t) BELLE_SIP_CAST(t,belle_sip_header_event_t)
+#define BELLE_SIP_EVENT "Event"
+
 BELLE_SIP_END_DECLS
 
 
