@@ -65,6 +65,7 @@ typedef unsigned int belle_sip_type_id_t;
 
 #define BELLE_SIP_VPTR_INIT(object_type,parent_type,unowned) \
 		BELLE_SIP_TYPE_ID(object_type), \
+		sizeof(object_type), \
 		#object_type,\
 		unowned,\
 		(belle_sip_object_get_vptr_t)BELLE_SIP_OBJECT_GET_VPTR_FUNC(parent_type), \
@@ -131,6 +132,7 @@ typedef struct _belle_sip_object_vptr *(*belle_sip_object_get_vptr_t)(void);
 
 struct _belle_sip_object_vptr{
 	belle_sip_type_id_t id;
+	size_t size; /*the size of the object - not the vptr size*/
 	const char *type_name;
 	int initially_unowned;
 	belle_sip_object_get_vptr_t get_parent;
@@ -145,7 +147,6 @@ typedef struct _belle_sip_object_vptr belle_sip_object_vptr_t;
 
 struct _belle_sip_object{
 	belle_sip_object_vptr_t *vptr;
-	size_t size;
 	int ref;
 	char* name;
 	struct weak_ref *weak_refs;
@@ -170,6 +171,26 @@ BELLESIP_EXPORT belle_sip_object_t * _belle_sip_object_new(size_t objsize, belle
 **/
 BELLESIP_EXPORT void belle_sip_object_enable_marshal_check(int enable);
 
+/**
+ * Activates an object leak detector. When enabled, belle-sip will reference all created objects.
+ * At program termination, application can check if objects remain alive using belle_sip_object_get_object_count() and dump them with
+ * belle_sip_object_dump_active_objects().
+ * @warning this must not be used in multi-threaded programs (when multiple threads can access belle-sip at the same time)
+ * Useful for debug purposes.
+ * @param enable TRUE to enable, FALSE to disable.
+**/
+BELLESIP_EXPORT void belle_sip_object_enable_leak_detector(int enable);
+
+BELLESIP_EXPORT int belle_sip_object_get_object_count(void);
+
+BELLESIP_EXPORT void belle_sip_object_dump_active_objects(void);
+
+/**
+ * Suspend leak detector from this point. If the leak detector wasn't activated, this function does nothing.
+ * This can be useful to make object allocation that have to remain active beyond the scope of a test.
+**/
+BELLESIP_EXPORT void belle_sip_object_inhibit_leak_detector(int yes);
+
 int belle_sip_object_is_unowed(const belle_sip_object_t *obj);
 
 /**
@@ -192,12 +213,12 @@ typedef void (*belle_sip_object_destroy_notify_t)(void *userpointer, belle_sip_o
  * This allows another object to be informed when object is destroyed, and then possibly
  * cleanups pointer it holds to this object.
 **/
-belle_sip_object_t *belle_sip_object_weak_ref(void *obj, belle_sip_object_destroy_notify_t destroy_notify, void *userpointer);
+BELLESIP_EXPORT belle_sip_object_t *belle_sip_object_weak_ref(void *obj, belle_sip_object_destroy_notify_t destroy_notify, void *userpointer);
 
 /**
  * Remove a weak reference to object.
 **/
-void belle_sip_object_weak_unref(void *obj, belle_sip_object_destroy_notify_t destroy_notify, void *userpointer);
+BELLESIP_EXPORT void belle_sip_object_weak_unref(void *obj, belle_sip_object_destroy_notify_t destroy_notify, void *userpointer);
 
 /**
  * Set object name.
@@ -303,7 +324,7 @@ char *belle_sip_object_describe(void *obj);
 /**
  * Returns a string describing the inheritance diagram and implemented interfaces of an object given its type name.
 **/
-char *belle_sip_object_describe_type_from_name(const char *name);
+BELLESIP_EXPORT char *belle_sip_object_describe_type_from_name(const char *name);
 
 BELLESIP_EXPORT void *belle_sip_object_cast(belle_sip_object_t *obj, belle_sip_type_id_t id, const char *castname, const char *file, int fileno);
 
@@ -467,6 +488,16 @@ BELLE_SIP_END_DECLS
  * 	BELLE_SIP_DECLARE_VPTR(belle_sip_dns_srv_t);
  */
 
-
+#if defined(__cplusplus) && defined(BELLE_SIP_USE_STL)
+#include <ostream>
+inline   std::ostream&
+operator<<( std::ostream& __os, const belle_sip_object_t* object)
+{
+	char* object_as_string = belle_sip_object_to_string((void*)object);
+	__os << object_as_string;
+	belle_sip_free(object_as_string);
+	return __os;
+}
+#endif
 #endif
 
