@@ -139,6 +139,17 @@ BELLESIP_EXPORT int belle_sip_stack_get_default_dscp(belle_sip_stack_t *stack);
 **/
 BELLESIP_EXPORT int belle_sip_stack_tls_available(belle_sip_stack_t *stack);
 
+/*
+ * returns timer config for this stack
+ * */
+BELLESIP_EXPORT const belle_sip_timer_config_t *belle_sip_stack_get_timer_config(const belle_sip_stack_t *stack);
+
+/*
+ *
+ * set sip timer config to be used for this stack
+ * */
+BELLESIP_EXPORT void belle_sip_stack_set_timer_config(belle_sip_stack_t *stack, const belle_sip_timer_config_t *timer_config);
+
 BELLE_SIP_END_DECLS
 
 #endif
