@@ -68,9 +68,9 @@ typedef enum {
 
 BELLE_SIP_BEGIN_DECLS
 
-extern belle_sip_log_function_t belle_sip_logv_out;
+BELLESIP_VAR_EXPORT belle_sip_log_function_t belle_sip_logv_out;
 
-extern unsigned int __belle_sip_log_mask;
+BELLESIP_VAR_EXPORT unsigned int __belle_sip_log_mask;
 
 #define belle_sip_log_level_enabled(level)   (__belle_sip_log_mask & (level))
 
@@ -153,8 +153,11 @@ static BELLESIP_INLINE void BELLE_SIP_CHECK_FORMAT_ARGS(1,2) belle_sip_fatal(con
 
 BELLESIP_EXPORT void belle_sip_set_log_file(FILE *file);
 BELLESIP_EXPORT void belle_sip_set_log_handler(belle_sip_log_function_t func);
+BELLESIP_EXPORT belle_sip_log_function_t belle_sip_get_log_handler();
 
 BELLESIP_EXPORT char * BELLE_SIP_CHECK_FORMAT_ARGS(1,2) belle_sip_strdup_printf(const char *fmt,...);
+BELLESIP_EXPORT char * belle_sip_strcat_vprintf(char* dst, const char *fmt, va_list ap);
+BELLESIP_EXPORT char * BELLE_SIP_CHECK_FORMAT_ARGS(2,3) belle_sip_strcat_printf(char* dst, const char *fmt,...);
 
 BELLESIP_EXPORT belle_sip_error_code BELLE_SIP_CHECK_FORMAT_ARGS(4,5) belle_sip_snprintf(char *buff, size_t buff_size, size_t *offset, const char *fmt, ...);
 BELLESIP_EXPORT belle_sip_error_code belle_sip_snprintf_valist(char *buff, size_t buff_size, size_t *offset, const char *fmt, va_list args);
