@@ -30,15 +30,14 @@ GET_SET_STRING(belle_sip_auth_event,passwd)
 GET_SET_STRING(belle_sip_auth_event,ha1)
 GET_SET_STRING(belle_sip_auth_event,distinguished_name)
 
-belle_sip_auth_event_t* belle_sip_auth_event_create(belle_sip_object_t *source, const char* realm, const belle_sip_header_from_t *from) {
+belle_sip_auth_event_t* belle_sip_auth_event_create(belle_sip_object_t *source, const char* realm, const belle_sip_uri_t *from_uri) {
 	belle_sip_auth_event_t* result = belle_sip_new0(belle_sip_auth_event_t);
 	result->source=source;
 	belle_sip_auth_event_set_realm(result,realm);
 	
-	if (from){
-		belle_sip_uri_t *uri=belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(from));
-		belle_sip_auth_event_set_username(result,belle_sip_uri_get_user(uri));
-		belle_sip_auth_event_set_domain(result,belle_sip_uri_get_host(uri));
+	if (from_uri){
+		belle_sip_auth_event_set_username(result,belle_sip_uri_get_user(from_uri));
+		belle_sip_auth_event_set_domain(result,belle_sip_uri_get_host(from_uri));
 	}
 	return result;
 }
@@ -73,9 +72,7 @@ belle_sip_signing_key_t* belle_sip_auth_event_get_signing_key(const belle_sip_au
 }
 
 void belle_sip_auth_event_set_signing_key(belle_sip_auth_event_t* event, belle_sip_signing_key_t* value) {
-	if (event->key) belle_sip_object_unref(event->key);
-	event->key=value;
-	if (event->key) belle_sip_object_ref(event->key);
+	SET_OBJECT_PROPERTY(event,key,value);
 }
 
 belle_sip_auth_mode_t belle_sip_auth_event_get_mode(const belle_sip_auth_event_t* event) {
@@ -122,4 +119,7 @@ void belle_tls_verify_policy_set_exceptions(belle_tls_verify_policy_t *obj, int
 	obj->exception_flags=flags;
 }
 
+unsigned int belle_tls_verify_policy_get_exceptions(const belle_tls_verify_policy_t *obj){
+	return obj->exception_flags;
+}
 
