@@ -76,6 +76,7 @@ static struct header_name_func_pair  header_table[] = {
 	,{PROTO_SIP, 			BELLE_SIP_DATE,					(header_parse_func)belle_sip_header_date_parse}
 	,{PROTO_SIP, 			BELLE_SIP_P_PREFERRED_IDENTITY,	(header_parse_func)belle_sip_header_p_preferred_identity_parse}
 	,{PROTO_SIP, 			BELLE_SIP_PRIVACY,				(header_parse_func)belle_sip_header_privacy_parse}
+	,{PROTO_SIP, 			BELLE_SIP_EVENT,				(header_parse_func)belle_sip_header_event_parse}
 };
 
 static belle_sip_header_t* belle_header_create(const char* name,const char* value,int protocol) {
@@ -83,7 +84,7 @@ static belle_sip_header_t* belle_header_create(const char* name,const char* valu
 	belle_sip_header_t* ret;
 	size_t elements =sizeof(header_table)/sizeof(struct header_name_func_pair);
 
-	if (!name || name[0]=='0') {
+	if (!name || name[0]=='\0') {
 		belle_sip_error("Cannot create header without name");
 		return NULL;
 	}
@@ -177,6 +178,7 @@ struct _belle_sip_header_address {
 	belle_sip_parameters_t base;
 	char* displayname;
 	belle_sip_uri_t* uri;
+	belle_generic_uri_t* absolute_uri;
 };
 
 static void belle_sip_header_address_init(belle_sip_header_address_t* object){
@@ -193,27 +195,40 @@ static void belle_sip_header_address_clone(belle_sip_header_address_t *addr, con
 	if (belle_sip_header_address_get_uri(orig)) {
 		belle_sip_header_address_set_uri(addr,BELLE_SIP_URI(belle_sip_object_clone(BELLE_SIP_OBJECT(belle_sip_header_address_get_uri(orig)))));
 	}
+	if (belle_sip_header_address_get_absolute_uri(orig)) {
+		belle_sip_header_address_set_absolute_uri(addr,BELLE_GENERIC_URI(belle_sip_object_clone(BELLE_SIP_OBJECT(belle_sip_header_address_get_absolute_uri(orig)))));
+	}
 }
 
 static belle_sip_error_code _belle_sip_header_address_marshal(belle_sip_header_address_t* header, char* buff, size_t buff_size, size_t *offset, int force_angle_quote) {
 	belle_sip_error_code error=BELLE_SIP_OK;
 	/*1 display name*/
 	if (header->displayname) {
-		error=belle_sip_snprintf(buff,buff_size,offset,"\"%s\" ",header->displayname);
+		char* escaped_display_name = belle_sip_display_name_to_backslashed_escaped_string(header->displayname);
+		error=belle_sip_snprintf(buff,buff_size,offset,"\"%s\" ",escaped_display_name);
+		belle_sip_free(escaped_display_name);
 		if (error!=BELLE_SIP_OK) return error;
 	}
-	if (header->uri) {
+	if (header->uri || header->absolute_uri) {
 		/*cases where < is required*/
-		if (force_angle_quote || header->displayname
+		if (force_angle_quote
+			|| header->displayname
+			|| header->absolute_uri
 			|| belle_sip_parameters_get_parameter_names((belle_sip_parameters_t*)header->uri)
 			|| belle_sip_uri_get_header_names(header->uri)
 			|| belle_sip_parameters_get_parameter_names(&header->base)) {
 			error=belle_sip_snprintf(buff,buff_size,offset,"%s","<");
 			if (error!=BELLE_SIP_OK) return error;
 		}
-		error=belle_sip_uri_marshal(header->uri,buff,buff_size,offset);
+		if (header->uri) {
+			error=belle_sip_uri_marshal(header->uri,buff,buff_size,offset);
+		} else {
+			error=belle_generic_uri_marshal(header->absolute_uri,buff,buff_size,offset);
+		}
 		if (error!=BELLE_SIP_OK) return error;
-		if (force_angle_quote || header->displayname
+		if (force_angle_quote
+				|| header->displayname
+				|| header->absolute_uri
 				|| belle_sip_parameters_get_parameter_names((belle_sip_parameters_t*)header->uri)
 				|| belle_sip_uri_get_header_names(header->uri)
 				|| belle_sip_parameters_get_parameter_names(&header->base)) {
@@ -247,11 +262,31 @@ belle_sip_uri_t* belle_sip_header_address_get_uri(const belle_sip_header_address
 }
 
 void belle_sip_header_address_set_uri(belle_sip_header_address_t* address, belle_sip_uri_t* uri) {
-	belle_sip_object_ref(uri);
+	if (uri) belle_sip_object_ref(uri);
 	if (address->uri){
 		belle_sip_object_unref(address->uri);
 	}
 	address->uri=uri;
+	if (address->absolute_uri && uri) {
+		belle_sip_warning("sip absolute uri [%p] already set for header_address [%p], cleaning it",address->absolute_uri, address);
+		belle_sip_header_address_set_absolute_uri(address,NULL);
+	}
+}
+
+belle_generic_uri_t* belle_sip_header_address_get_absolute_uri(const belle_sip_header_address_t* address) {
+	return address->absolute_uri;
+}
+
+void belle_sip_header_address_set_absolute_uri(belle_sip_header_address_t* address, belle_generic_uri_t* absolute_uri) {
+	belle_sip_object_ref(absolute_uri);
+	if (address->absolute_uri){
+		belle_sip_object_unref(address->absolute_uri);
+	}
+	address->absolute_uri=absolute_uri;
+	if (address->uri && absolute_uri) {
+		belle_sip_warning("sip uri [%p] already set for header_address [%p], cleaning it",address->uri, address);
+		belle_sip_header_address_set_uri(address,NULL);
+	}
 }
 
 belle_sip_header_address_t* belle_sip_header_address_create(const char* display, belle_sip_uri_t* uri) {
@@ -261,6 +296,13 @@ belle_sip_header_address_t* belle_sip_header_address_create(const char* display,
 	return address;
 }
 
+belle_sip_header_address_t* belle_sip_header_address_create2(const char* display, belle_generic_uri_t* uri) {
+	belle_sip_header_address_t* address = belle_sip_header_address_new();
+	belle_sip_header_address_set_displayname(address,display);
+	belle_sip_header_address_set_absolute_uri(address,uri);
+	return address;
+}
+
 /******************************
  * Extension header inherits from header
  *
@@ -571,11 +613,15 @@ void belle_sip_header_user_agent_set_products(belle_sip_header_user_agent_t* use
 void belle_sip_header_user_agent_add_product(belle_sip_header_user_agent_t* user_agent,const char* product) {
 	user_agent->products = belle_sip_list_append(user_agent->products ,belle_sip_strdup(product));
 }
+
 int belle_sip_header_user_agent_get_products_as_string(const belle_sip_header_user_agent_t* user_agent,char* value,unsigned int value_size) {
 	size_t result = 0;
+	belle_sip_error_code error=BELLE_SIP_OK;
 	belle_sip_list_t* list = user_agent->products;
+	
 	for(;list!=NULL;list=list->next){
-		belle_sip_snprintf(value,value_size,&result,"%s ",(const char *)list->data);
+		error=belle_sip_snprintf(value,value_size,&result,"%s ",(const char *)list->data);
+		if (error!=BELLE_SIP_OK) return -1;
 	}
 	if (result>0) value[result]='\0'; /*remove last space */
 
@@ -991,7 +1037,7 @@ static void belle_sip_header_extension_clone(belle_sip_header_extension_t* exten
 belle_sip_error_code belle_sip_header_extension_marshal(belle_sip_header_extension_t* extension, char* buff, size_t buff_size, size_t *offset) {
 	belle_sip_error_code error=belle_sip_header_marshal(BELLE_SIP_HEADER(extension), buff, buff_size, offset);
 	if (error!=BELLE_SIP_OK) return error;
-	error=belle_sip_snprintf(buff,buff_size,offset,"%s",extension->value);
+	if (extension->value) error=belle_sip_snprintf(buff,buff_size,offset,"%s",extension->value);
 	if (error!=BELLE_SIP_OK) return error;
 	return error;
 }
@@ -1612,19 +1658,22 @@ static const char *days[]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
 static const char *months[]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
 
 BELLESIP_EXPORT time_t belle_sip_header_date_get_time(belle_sip_header_date_t *obj){
-	struct tm ret={0};
-	char tmp1[16]={0};
-	char tmp2[16]={0};
+	struct tm ret ={0};
+	char tmp1[16] ={0};
+	char tmp2[16] ={0};
 	int i,j;
 	time_t seconds;
-	
+	time_t adjust_timezone;
+
+
+	/* time headers are in GMT as spec says */
 	sscanf(obj->date,"%3c,%d %16s %d %d:%d:%d",tmp1,&ret.tm_mday,tmp2,
 		&ret.tm_year,&ret.tm_hour,&ret.tm_min,&ret.tm_sec);
 	ret.tm_year-=1900;
-	for(i=0;i<7;i++) { 
+	for(i=0;i<7;i++) {
 		if(strcmp(tmp1,days[i])==0) {
 			ret.tm_wday=i;
-			for(j=0;j<12;j++) { 
+			for(j=0;j<12;j++) {
 				if(strcmp(tmp2,months[j])==0) {
 					ret.tm_mon=j;
 					goto success;
@@ -1635,18 +1684,27 @@ BELLESIP_EXPORT time_t belle_sip_header_date_get_time(belle_sip_header_date_t *o
 	belle_sip_warning("Failed to parse date %s",obj->date);
 	return (time_t)-1;
 success:
-	
 	ret.tm_isdst=0;
-	seconds=mktime(&ret);
+
+#if TARGET_IPHONE_SIMULATOR
+	/* 'timezone' is buggy on iOS simulator, use the timegm() function to convert to UTC timestamp
+	   and discard the adjust timezone value */
+	seconds = timegm(&ret);
+	adjust_timezone = 0;
+#else
+	seconds = mktime(&ret);
+	adjust_timezone = timezone;
+#endif
+
 	if (seconds==(time_t)-1){
 		belle_sip_error("mktime() failed: %s",strerror(errno));
 		return (time_t)-1;
 	}
-	return seconds-timezone;
+	return seconds-adjust_timezone;
 }
 
 BELLESIP_EXPORT void belle_sip_header_date_set_time(belle_sip_header_date_t *obj, const time_t *utc_time){
-	
+
 	struct tm *ret;
 #ifndef WIN32
 	struct tm gmt;
@@ -1737,3 +1795,43 @@ belle_sip_header_privacy_t* belle_sip_header_privacy_create(const char* privacy)
 	belle_sip_header_privacy_add_privacy(privacy_header,privacy);
 	return privacy_header;
 }
+
+/**************************
+* Event header object inherent from parameters
+****************************
+*/
+struct _belle_sip_header_event  {
+	belle_sip_parameters_t parameters;
+	const char* package_name;
+};
+
+static void belle_sip_header_event_destroy(belle_sip_header_event_t* event) {
+	DESTROY_STRING(event,package_name);
+}
+
+static void belle_sip_header_event_clone(belle_sip_header_event_t* event, const belle_sip_header_event_t *orig ) {
+	CLONE_STRING(belle_sip_header_event,package_name,event,orig)
+}
+
+belle_sip_error_code belle_sip_header_event_marshal(belle_sip_header_event_t* event, char* buff, size_t buff_size, size_t *offset) {
+	belle_sip_error_code error=belle_sip_header_marshal(BELLE_SIP_HEADER(event), buff, buff_size, offset);
+	if (error!=BELLE_SIP_OK) return error;
+	error=belle_sip_snprintf(buff,buff_size,offset,"%s",event->package_name);
+	if (error!=BELLE_SIP_OK) return error;
+	error=belle_sip_parameters_marshal(BELLE_SIP_PARAMETERS(event), buff, buff_size, offset);
+	if (error!=BELLE_SIP_OK) return error;
+	return error;
+}
+
+BELLE_SIP_NEW_HEADER(header_event,parameters,BELLE_SIP_EVENT)
+BELLE_SIP_PARSE(header_event)
+GET_SET_STRING(belle_sip_header_event,package_name);
+GET_SET_STRING_PARAM(belle_sip_header_event,id);
+
+belle_sip_header_event_t* belle_sip_header_event_create (const char* package_name)  {
+	belle_sip_header_event_t* event=belle_sip_header_event_new();
+	belle_sip_header_event_set_package_name(event,package_name);
+	return event;
+}
+
+
