@@ -109,7 +109,7 @@
 #define BELLE_SIP_INVOKE_LISTENER_ARG(listener,interface_name,method,arg) \
 	((BELLE_SIP_INTERFACE_GET_METHODS((listener),interface_name)->method!=NULL)  ? \
 		BELLE_SIP_INTERFACE_GET_METHODS((listener),interface_name)->method(listener,(arg)) : 0 )
-			
+
 typedef struct weak_ref{
 	struct weak_ref *next;
 	belle_sip_object_destroy_notify_t notify;
@@ -169,7 +169,10 @@ BELLE_SIP_DECLARE_VPTR(belle_sdp_media_t);
 BELLE_SIP_DECLARE_VPTR(belle_sdp_media_description_t);
 BELLE_SIP_DECLARE_VPTR(belle_sdp_origin_t);
 BELLE_SIP_DECLARE_VPTR(belle_sdp_phone_t);
+BELLE_SIP_DECLARE_VPTR(belle_sdp_raw_attribute_t);
 BELLE_SIP_DECLARE_VPTR(belle_sdp_repeate_time_t);
+BELLE_SIP_DECLARE_VPTR(belle_sdp_rtcp_fb_attribute_t);
+BELLE_SIP_DECLARE_VPTR(belle_sdp_rtcp_xr_attribute_t);
 BELLE_SIP_DECLARE_VPTR(belle_sdp_session_description_t);
 BELLE_SIP_DECLARE_VPTR(belle_sdp_session_name_t);
 BELLE_SIP_DECLARE_VPTR(belle_sdp_time_t);
@@ -201,6 +204,7 @@ BELLE_SIP_DECLARE_VPTR(belle_generic_uri_t);
 BELLE_SIP_DECLARE_VPTR(belle_http_callbacks_t);
 BELLE_SIP_DECLARE_VPTR(belle_tls_verify_policy_t);
 BELLE_SIP_DECLARE_VPTR(belle_http_header_authorization_t);
+BELLE_SIP_DECLARE_VPTR(belle_sip_header_event_t);
 
 BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_resolver_context_t,belle_sip_source_t)
 	void (*cancel)(belle_sip_resolver_context_t *);
@@ -223,24 +227,29 @@ struct belle_sip_source{
 	unsigned long id;
 	belle_sip_fd_t fd;
 	unsigned short events,revents;
+#ifdef WIN32
+	unsigned short armed_events;
+	unsigned short pad;
+#endif
 	int timeout;
 	void *data;
 	uint64_t expire_ms;
 	int index; /* index in pollfd table */
 	belle_sip_source_func_t notify;
 	belle_sip_source_remove_callback_t on_remove;
+	belle_sip_socket_t sock;
 	unsigned char cancelled;
 	unsigned char expired;
 	unsigned char oneshot;
 	unsigned char notify_required; /*for testing purpose, use to ask for being scheduled*/
-	belle_sip_socket_t sock;
 };
 
 void belle_sip_socket_source_init(belle_sip_source_t *s, belle_sip_source_func_t func, void *data, belle_sip_socket_t fd, unsigned int events, unsigned int timeout_value_ms);
 void belle_sip_fd_source_init(belle_sip_source_t *s, belle_sip_source_func_t func, void *data, belle_sip_fd_t fd, unsigned int events, unsigned int timeout_value_ms);
 void belle_sip_source_uninit(belle_sip_source_t *s);
+void belle_sip_source_set_notify(belle_sip_source_t *s, belle_sip_source_func_t func);
+
 
-#define belle_list_next(elem) ((elem)->next)
 
 /* include private headers */
 #include "channel.h"
@@ -251,9 +260,11 @@ void belle_sip_source_uninit(belle_sip_source_t *s);
 #define belle_sip_new(type) (type*)belle_sip_malloc(sizeof(type))
 #define belle_sip_new0(type) (type*)belle_sip_malloc0(sizeof(type))
 
+#define belle_list_next(elem) ((elem)->next)
 belle_sip_list_t *belle_sip_list_new(void *data);
 belle_sip_list_t*  belle_sip_list_append_link(belle_sip_list_t* elem,belle_sip_list_t *new_elem);
 belle_sip_list_t *belle_sip_list_delete_custom(belle_sip_list_t *list, belle_sip_compare_func compare_func, const void *user_data);
+belle_sip_list_t*  _belle_sip_list_remove(belle_sip_list_t* first, void *data, int warn_if_not_found);
 
 #define belle_sip_list_next(elem) ((elem)->next)
 
@@ -272,10 +283,6 @@ struct belle_sip_dict {
 
 BELLESIP_INTERNAL_EXPORT char * belle_sip_concat (const char *str, ...);
 
-BELLESIP_INTERNAL_EXPORT uint64_t belle_sip_time_ms(void);
-
-BELLESIP_INTERNAL_EXPORT unsigned int belle_sip_random(void);
-
 
 /*parameters accessors*/
 #define GET_SET_STRING(object_type,attribute) \
@@ -283,11 +290,12 @@ BELLESIP_INTERNAL_EXPORT unsigned int belle_sip_random(void);
 		return obj->attribute;\
 	}\
 	void object_type##_set_##attribute (object_type##_t* obj,const char* value) {\
-		if (obj->attribute != NULL) belle_sip_free((void*)obj->attribute);\
+		const char* previous_value = obj->attribute;  /*preserve if same value re-asigned*/ \
 		if (value) {\
 			obj->attribute=belle_sip_strdup(value); \
 		} else obj->attribute=NULL;\
-	}
+		if (previous_value != NULL) belle_sip_free((void*)previous_value);\
+}
 /*#define GET_SET_STRING_PARAM_NULL_ALLOWED(object_type,attribute) \
 	GET_STRING_PARAM2(object_type,attribute,attribute) \
 	void object_type##_set_##func_name (object_type##_t* obj,const char* value) {\
@@ -519,7 +527,6 @@ belle_sip_hop_t* belle_sip_hop_new_from_uri(const belle_sip_uri_t *uri);
 belle_sip_hop_t* belle_sip_hop_new_from_generic_uri(const belle_generic_uri_t *uri);
 
 belle_sip_hop_t * belle_sip_stack_get_next_hop(belle_sip_stack_t *stack, belle_sip_request_t *req);
-const belle_sip_timer_config_t *belle_sip_stack_get_timer_config(const belle_sip_stack_t *stack);
 
 /*
  belle_sip_provider_t
@@ -540,13 +547,12 @@ struct belle_sip_provider{
 	unsigned char unconditional_answer_enabled;
 };
 
-belle_sip_provider_t *belle_sip_provider_new(belle_sip_stack_t *s, belle_sip_listening_point_t *lp);
+BELLESIP_INTERNAL_EXPORT belle_sip_provider_t *belle_sip_provider_new(belle_sip_stack_t *s, belle_sip_listening_point_t *lp);
 void belle_sip_provider_add_client_transaction(belle_sip_provider_t *prov, belle_sip_client_transaction_t *t);
 belle_sip_client_transaction_t *belle_sip_provider_find_matching_client_transaction(belle_sip_provider_t *prov, belle_sip_response_t *resp);
 void belle_sip_provider_remove_client_transaction(belle_sip_provider_t *prov, belle_sip_client_transaction_t *t);
 void belle_sip_provider_add_server_transaction(belle_sip_provider_t *prov, belle_sip_server_transaction_t *t);
-belle_sip_server_transaction_t * belle_sip_provider_find_matching_server_transaction(belle_sip_provider_t *prov,
-																				   belle_sip_request_t *req);
+BELLESIP_INTERNAL_EXPORT belle_sip_server_transaction_t * belle_sip_provider_find_matching_server_transaction(belle_sip_provider_t *prov,belle_sip_request_t *req);
 void belle_sip_provider_remove_server_transaction(belle_sip_provider_t *prov, belle_sip_server_transaction_t *t);
 void belle_sip_provider_set_transaction_terminated(belle_sip_provider_t *p, belle_sip_transaction_t *t);
 belle_sip_channel_t * belle_sip_provider_get_channel(belle_sip_provider_t *p, const belle_sip_hop_t *hop);
@@ -555,9 +561,10 @@ void belle_sip_provider_remove_dialog(belle_sip_provider_t *prov, belle_sip_dial
 void belle_sip_provider_release_channel(belle_sip_provider_t *p, belle_sip_channel_t *chan);
 void belle_sip_provider_add_internal_sip_listener(belle_sip_provider_t *p, belle_sip_listener_t *l, int prepend);
 void belle_sip_provider_remove_internal_sip_listener(belle_sip_provider_t *p, belle_sip_listener_t *l);
-belle_sip_client_transaction_t * belle_sip_provider_find_matching_client_transaction_from_req(belle_sip_provider_t *prov, belle_sip_request_t *req) ;
+belle_sip_client_transaction_t * belle_sip_provider_find_matching_client_transaction_from_req(belle_sip_provider_t *prov, belle_sip_request_t *req);
+belle_sip_dialog_t *belle_sip_provider_find_dialog_from_message(belle_sip_provider_t *prov, belle_sip_message_t *msg, int as_uas);
 /*for testing purpose only:*/
-void belle_sip_provider_dispatch_message(belle_sip_provider_t *prov, belle_sip_message_t *msg);
+BELLESIP_INTERNAL_EXPORT void belle_sip_provider_dispatch_message(belle_sip_provider_t *prov, belle_sip_message_t *msg);
 
 typedef struct listener_ctx{
 	belle_sip_listener_t *listener;
@@ -579,20 +586,21 @@ belle_http_provider_t *belle_http_provider_new(belle_sip_stack_t *s, const char
 /*
  * SIP and http messages
 **/
+#define BELLESIP_MULTIPART_BOUNDARY "---------------------------14737809831466499882746641449"
 
 void belle_sip_message_init(belle_sip_message_t *message);
 
 struct _belle_sip_message {
 	belle_sip_object_t base;
 	belle_sip_list_t* header_list;
-	char* body;
-	unsigned int body_length;
+	belle_sip_body_handler_t *body_handler;
 };
 
 struct _belle_sip_request {
-	belle_sip_message_t message;
+	belle_sip_message_t base;
 	char* method;
 	belle_sip_uri_t* uri;
+	belle_generic_uri_t *absolute_uri;
 	belle_sip_dialog_t *dialog;/*set if request was created by a dialog to avoid to search in dialog list*/
 	char *rfc2543_branch; /*computed 'branch' id in case we receive this request from an old RFC2543 stack*/
 	unsigned char dialog_queued;
@@ -608,10 +616,14 @@ struct belle_http_request{
 	belle_http_request_listener_t *listener;
 	belle_generic_uri_t *orig_uri;/*original uri before removing host and user/passwd*/
 	belle_http_response_t *response;
+	belle_sip_channel_t *channel;
 	int auth_attempt_count;
+	int background_task_id;
+	int cancelled;
 };
 
 void belle_http_request_set_listener(belle_http_request_t *req, belle_http_request_listener_t *l);
+void belle_http_request_set_channel(belle_http_request_t *req, belle_sip_channel_t *chan);
 void belle_http_request_set_response(belle_http_request_t *req, belle_http_response_t *resp);
 /*
  belle_sip_transaction_t
@@ -629,6 +641,7 @@ struct belle_sip_transaction{
 	void *appdata;
 	unsigned char is_internal;
 	unsigned char timed_out;
+	unsigned char sent_by_dialog_queue;
 };
 
 
@@ -910,7 +923,7 @@ struct belle_sip_auth_event {
 	belle_sip_signing_key_t* key;
 };
 
-belle_sip_auth_event_t* belle_sip_auth_event_create(belle_sip_object_t *source, const char* realm,const belle_sip_header_from_t * from);
+belle_sip_auth_event_t* belle_sip_auth_event_create(belle_sip_object_t *source, const char* realm,const belle_sip_uri_t * from_uri);
 
 void belle_sip_auth_event_set_distinguished_name(belle_sip_auth_event_t* event,const char* value);
 
@@ -969,8 +982,56 @@ belle_sip_header_extension_t* belle_sip_header_extension_new();
 
 belle_sip_header_extension_t* belle_sip_header_extension_parse (const char* extension) ;
 belle_sip_header_extension_t* belle_sip_header_extension_create (const char* name,const char* value);
-const char* belle_sip_header_extension_get_value(const belle_sip_header_extension_t* extension);
+BELLESIP_INTERNAL_EXPORT const char* belle_sip_header_extension_get_value(const belle_sip_header_extension_t* extension);
 void belle_sip_header_extension_set_value(belle_sip_header_extension_t* extension,const char* value);
 #define BELLE_SIP_HEADER_EXTENSION(t) BELLE_SIP_CAST(t,belle_sip_header_extension_t)
 
+
+/****************
+ * belle_sip_body_handler_t object
+ ***************/
+
+
+BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_body_handler_t,belle_sip_object_t)
+	void (*chunk_recv)(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, size_t offset, const uint8_t *buf, size_t size);
+	int (*chunk_send)(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, size_t offset, uint8_t *buf, size_t * size);
+BELLE_SIP_DECLARE_CUSTOM_VPTR_END
+
+void belle_sip_body_handler_begin_transfer(belle_sip_body_handler_t *obj);
+void belle_sip_body_handler_recv_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, const uint8_t *buf, size_t size);
+int belle_sip_body_handler_send_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, uint8_t *buf, size_t *size);
+void belle_sip_body_handler_end_transfer(belle_sip_body_handler_t *obj);
+
+
+BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_memory_body_handler_t,belle_sip_body_handler_t)
+BELLE_SIP_DECLARE_CUSTOM_VPTR_END
+
+BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_user_body_handler_t,belle_sip_body_handler_t)
+BELLE_SIP_DECLARE_CUSTOM_VPTR_END
+
+BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_file_body_handler_t,belle_sip_body_handler_t)
+BELLE_SIP_DECLARE_CUSTOM_VPTR_END
+
+BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_multipart_body_handler_t,belle_sip_body_handler_t)
+BELLE_SIP_DECLARE_CUSTOM_VPTR_END
+
+/**
+ * file manipulation
+ */
+/**
+ * Parse a directory and return all files in it.
+ *
+ * @param[in]	path		The directory to be parsed
+ * @param[in]	file_type	if not NULL return only the file with the given extension, must include the '.', ex:".pem"
+ * @return		a belle_sip list containing all found file or NULL if no file were found or directory doesn't exist. List must be destroyed using belle_sip_list_free_with_data(<ret_list>, belle_sip_free)
+ */
+belle_sip_list_t *belle_sip_parse_directory(const char *path, const char *file_type); 
+
+/**
+ * create a directory if it doesn't already exists
+ *
+ * @param[in] 	path		The directory to be created
+ * @return 0 in case of succes, -1 otherwise, note it returns -1 if the directory already exists
+ */
+int belle_sip_mkdir(const char *path);
 #endif
