@@ -89,12 +89,14 @@ static void belle_sip_source_to_poll(belle_sip_source_t *s, belle_sip_pollfd_t *
 		long events=0;
 		
 		if (s->events & BELLE_SIP_EVENT_READ)
-			events|=FD_READ;
+			events|=FD_READ|FD_ACCEPT;
 		if (s->events & BELLE_SIP_EVENT_WRITE)
-			events|=FD_WRITE;
-		
-		err=WSAEventSelect(s->sock,s->fd,events);
-		if (err!=0) belle_sip_error("WSAEventSelect() failed: %s",belle_sip_get_socket_error_string());
+			events|=FD_WRITE|FD_CONNECT;
+		if (events!=s->armed_events){
+			s->armed_events=events;
+			err=WSAEventSelect(s->sock,s->fd,events);
+			if (err!=0) belle_sip_error("WSAEventSelect() failed: %s",belle_sip_get_socket_error_string());
+		}
 	}
 }
 
@@ -103,19 +105,22 @@ static unsigned int belle_sip_source_get_revents(belle_sip_source_t *s,belle_sip
 	int err;
 	unsigned int ret=0;
 	
-	/*special treatments for windows sockets*/
-	if (s->sock!=(belle_sip_socket_t)-1){
-		err=WSAEnumNetworkEvents(s->sock,NULL,&revents);
-		if (err!=0){
-			belle_sip_error("WSAEnumNetworkEvents() failed: %s socket=%x",belle_sip_get_socket_error_string(),(unsigned int)s->sock);
-			return 0;
-		}
-		if (revents.lNetworkEvents & FD_READ)
-			ret|=BELLE_SIP_EVENT_READ;
-		if (revents.lNetworkEvents & FD_WRITE)
-			ret|=BELLE_SIP_EVENT_WRITE;
-	}else{
-		if (WaitForSingleObjectEx(s->fd,0,FALSE)==WAIT_OBJECT_0){
+	if (WaitForSingleObjectEx(s->fd,0,FALSE)==WAIT_OBJECT_0){
+		if (s->sock!=(belle_sip_socket_t)-1){
+			/*special treatments for windows sockets*/
+			err=WSAEnumNetworkEvents(s->sock,s->fd,&revents);
+			if (err!=0){
+				belle_sip_error("WSAEnumNetworkEvents() failed: %s socket=%x",belle_sip_get_socket_error_string(),(unsigned int)s->sock);
+				return 0;
+			}
+			if (revents.lNetworkEvents & FD_READ || revents.lNetworkEvents & FD_ACCEPT){
+				ret|=BELLE_SIP_EVENT_READ;
+			}
+			if (revents.lNetworkEvents & FD_WRITE || revents.lNetworkEvents & FD_CONNECT){
+				ret|=BELLE_SIP_EVENT_WRITE;
+			}
+			s->armed_events=0;
+		}else{
 			ret=BELLE_SIP_EVENT_READ;
 			ResetEvent(s->fd);
 		}
@@ -174,6 +179,10 @@ void belle_sip_source_uninit(belle_sip_source_t *obj){
 	obj->sock=(belle_sip_socket_t)-1;
 }
 
+void belle_sip_source_set_notify(belle_sip_source_t *s, belle_sip_source_func_t func) {
+	s->notify = func;
+}
+
 void belle_sip_socket_source_init(belle_sip_source_t *s, belle_sip_source_func_t func, void *data, belle_sip_socket_t sock, unsigned int events, unsigned int timeout_value_ms){
 #ifdef WIN32
 	/*on windows, the fd to poll is not the socket */
@@ -235,6 +244,7 @@ struct belle_sip_main_loop{
 	belle_sip_object_pool_t *pool;
 	int nsources;
 	int run;
+	int in_iterate;
 };
 
 void belle_sip_main_loop_remove_source(belle_sip_main_loop_t *ml, belle_sip_source_t *source){
@@ -355,6 +365,12 @@ void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
 	int can_clean=belle_sip_object_pool_cleanable(ml->pool); /*iterate might not be called by the thread that created the main loop*/ 
 	belle_sip_object_pool_t *tmp_pool=NULL;
 	
+	if (ml->in_iterate){
+		belle_sip_warning("belle_sip_main_loop_iterate([%p]): reentrancy detected, doing nothing.",ml);
+		return;
+	}
+	ml->in_iterate=TRUE;
+	
 	if (!can_clean){
 		/*Push a temporary pool for the time of the iterate loop*/
 		tmp_pool=belle_sip_object_pool_push();
@@ -424,7 +440,10 @@ void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
 		next=elem->next;
 		if (!s->cancelled){
 			char *objdesc=belle_sip_object_to_string((belle_sip_object_t*)s);
-			if (s->timeout>0)/*to avoid too many traces*/ belle_sip_debug("source %s notified revents=%u, timeout=%i",objdesc,revents,s->timeout);
+			if (s->timeout>0) {
+				/*to avoid too many traces*/ 
+				belle_sip_debug("source %s notified revents=%u, timeout=%i",objdesc,revents,s->timeout); 
+			}
 			belle_sip_free(objdesc);
 			ret=s->notify(s->data,s->revents);
 			if (ret==BELLE_SIP_STOP || s->oneshot){
@@ -449,12 +468,15 @@ void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
 	else if (tmp_pool) belle_sip_object_unref(tmp_pool);
 end:
 	belle_sip_free(pfd);
+	
+	ml->in_iterate=FALSE;
 }
 
 void belle_sip_main_loop_run(belle_sip_main_loop_t *ml){
 	ml->run=1;
 	while(ml->run){
 		belle_sip_main_loop_iterate(ml);
+		if (ml->in_iterate) break;
 	}
 }
 
