@@ -40,15 +40,59 @@ void belle_sip_object_enable_marshal_check(int enable) {
 	_belle_sip_object_marshal_check_enabled = (enable) ? TRUE : FALSE;
 }
 
+
+static belle_sip_list_t *all_objects=NULL;
+static int belle_sip_leak_detector_enabled=FALSE;
+static int belle_sip_leak_detector_inhibited=FALSE;
+
+static void add_new_object(belle_sip_object_t *obj){
+	if (belle_sip_leak_detector_enabled && !belle_sip_leak_detector_inhibited){
+		all_objects=belle_sip_list_prepend(all_objects,obj);
+	}
+}
+
+static void remove_free_object(belle_sip_object_t *obj){
+	if (belle_sip_leak_detector_enabled && !belle_sip_leak_detector_inhibited){
+		all_objects=_belle_sip_list_remove(all_objects,obj,FALSE); /*it may fail if the leak detector was inhibitted at the time the object was created*/
+	}
+}
+
+void belle_sip_object_inhibit_leak_detector(int yes){
+	belle_sip_leak_detector_inhibited=yes ? TRUE : FALSE;
+}
+
+void belle_sip_object_enable_leak_detector(int enable){
+	belle_sip_leak_detector_enabled=enable;
+}
+
+int belle_sip_object_get_object_count(void){
+	return belle_sip_list_size(all_objects);
+}
+
+void belle_sip_object_dump_active_objects(void){
+	belle_sip_list_t *elem;
+	
+	if (all_objects){
+		belle_sip_message("List of leaked objects:");
+		for(elem=all_objects;elem!=NULL;elem=elem->next){
+			belle_sip_object_t *obj=(belle_sip_object_t*)elem->data;
+			char* content= belle_sip_object_to_string(obj);
+			belle_sip_message("%s(%p) ref=%i, content [%10s...]",obj->vptr->type_name,obj,obj->ref,content);
+			belle_sip_free(content);
+		}
+	}else belle_sip_message("No objects leaked.");
+}
+
 belle_sip_object_t * _belle_sip_object_new(size_t objsize, belle_sip_object_vptr_t *vptr){
-	belle_sip_object_t *obj=(belle_sip_object_t *)belle_sip_malloc0(objsize);
+	belle_sip_object_t *obj=(belle_sip_object_t *)belle_sip_malloc0(vptr->size);
+
 	obj->ref=vptr->initially_unowned ? 0 : 1;
 	obj->vptr=vptr;
-	obj->size=objsize;
 	if (obj->ref==0){
 		belle_sip_object_pool_t *pool=belle_sip_object_pool_get_current();
 		if (pool) belle_sip_object_pool_add(pool,obj);
 	}
+	add_new_object(obj);
 	return obj;
 }
 
@@ -106,6 +150,7 @@ belle_sip_object_t *belle_sip_object_weak_ref(void *obj, belle_sip_object_destro
 void belle_sip_object_weak_unref(void *obj, belle_sip_object_destroy_notify_t destroy_notify, void *userpointer){
 	belle_sip_object_t *o=BELLE_SIP_OBJECT(obj);
 	weak_ref_t *ref,*prevref=NULL,*next=NULL;
+	int found=FALSE;
 
 	if (o->ref==-1) return; /*too late and avoid recursions*/
 	for(ref=o->weak_refs;ref!=NULL;ref=next){
@@ -114,12 +159,13 @@ void belle_sip_object_weak_unref(void *obj, belle_sip_object_destroy_notify_t de
 			if (prevref==NULL) o->weak_refs=next;
 			else prevref->next=next;
 			belle_sip_free(ref);
-			return;
+			found=TRUE;
+			/*do not break or return, someone could have put twice the same weak ref on the same object*/
 		}else{
 			prevref=ref;
 		}
 	}
-	belle_sip_fatal("Could not find weak_ref, you're a looser.");
+	if (!found) belle_sip_fatal("Could not find weak_ref, you're a looser.");
 }
 
 static void belle_sip_object_loose_weak_refs(belle_sip_object_t *obj){
@@ -151,6 +197,7 @@ static belle_sip_object_vptr_t *no_parent(void){
 
 belle_sip_object_vptr_t belle_sip_object_t_vptr={
 	BELLE_SIP_TYPE_ID(belle_sip_object_t),
+	sizeof(belle_sip_object_t),
 	"belle_sip_object_t",
 	FALSE,
 	no_parent, /*no parent, it's god*/
@@ -169,6 +216,7 @@ void belle_sip_object_delete(void *ptr){
 	belle_sip_object_vptr_t *vptr;
 
 	belle_sip_object_loose_weak_refs(obj);
+	remove_free_object(obj);
 	vptr=obj->vptr;
 	while(vptr!=NULL){
 		if (vptr->destroy) vptr->destroy(obj);
@@ -210,15 +258,15 @@ void _belle_sip_object_copy(belle_sip_object_t *newobj, const belle_sip_object_t
 belle_sip_object_t *belle_sip_object_clone(const belle_sip_object_t *obj){
 	belle_sip_object_t *newobj;
 
-	newobj=belle_sip_malloc0(obj->size);
+	newobj=belle_sip_malloc0(obj->vptr->size);
 	newobj->ref=obj->vptr->initially_unowned ? 0 : 1;
 	newobj->vptr=obj->vptr;
-	newobj->size=obj->size;
 	_belle_sip_object_copy(newobj,obj);
 	if (newobj->ref==0){
 		belle_sip_object_pool_t *pool=belle_sip_object_pool_get_current();
 		if (pool) belle_sip_object_pool_add(pool,newobj);
 	}
+	add_new_object(newobj);
 	return newobj;
 }
 
@@ -242,8 +290,9 @@ static int belle_sip_object_data_find(const void* a, const void* b)
 static void belle_sip_object_data_destroy(void* data)
 {
 	struct belle_sip_object_data* da = (struct belle_sip_object_data*)data;
-	if(da->destroy_func ) da->destroy_func(da->data);
+	if (da->destroy_func) da->destroy_func(da->data);
 	belle_sip_free(da->name);
+	belle_sip_free(da);
 }
 
 int belle_sip_object_data_set( belle_sip_object_t *obj, const char* name, void* data, belle_sip_data_destroy destroy_func )
@@ -745,4 +794,3 @@ belle_sip_object_pool_t *belle_sip_object_pool_get_current(void){
 }
 
 
-
