@@ -122,6 +122,19 @@ struct belle_sip_simple_resolver_context{
 	int family;
 	int flags;
 	uint64_t start_time;
+#ifdef USE_GETADDRINFO_FALLBACK
+	belle_sip_source_t *getaddrinfo_source;
+	belle_sip_thread_t getaddrinfo_thread;
+	belle_sip_mutex_t getaddrinfo_mutex;
+	unsigned char getaddrinfo_done;
+	unsigned char getaddrinfo_cancelled;
+	unsigned char getaddrinfo_notified;
+#ifdef WIN32
+	HANDLE ctlevent;
+#else
+	int ctlpipe[2];
+#endif
+#endif
 };
 
 struct belle_sip_combined_resolver_context{
@@ -396,7 +409,13 @@ static void append_dns_result(belle_sip_simple_resolver_context_t *ctx, struct s
 		return;
 	}
 	if (ctx->flags & AI_V4MAPPED) family=AF_INET6;
+#ifdef USE_GETADDRINFO_FALLBACK
+	belle_sip_mutex_lock(&ctx->getaddrinfo_mutex);
+#endif
 	ctx->ai_list = ai_list_append(ctx->ai_list, belle_sip_ip_address_to_addrinfo(family, host, ctx->port));
+#ifdef USE_GETADDRINFO_FALLBACK
+	belle_sip_mutex_unlock(&ctx->getaddrinfo_mutex);
+#endif
 	belle_sip_message("%s resolved to %s", ctx->name, host);
 }
 
@@ -416,7 +435,13 @@ static int resolver_process_data(belle_sip_simple_resolver_context_t *ctx, unsig
 	
 	if (simulated_timeout || ((revents & BELLE_SIP_EVENT_TIMEOUT) && (belle_sip_time_ms()-ctx->start_time>=timeout))) {
 		belle_sip_error("%s timed-out", __FUNCTION__);
+#ifdef USE_GETADDRINFO_FALLBACK
+		if (!ctx->getaddrinfo_notified) {
+			notify_results(ctx);
+		}
+#else
 		notify_results(ctx);
+#endif
 		return BELLE_SIP_STOP;
 	}
 	/*belle_sip_message("resolver_process_data(): revents=%i",revents);*/
@@ -463,21 +488,102 @@ static int resolver_process_data(belle_sip_simple_resolver_context_t *ctx, unsig
 			}
 		}
 		free(ans);
+#ifdef USE_GETADDRINFO_FALLBACK
+		if (!ctx->getaddrinfo_notified) {
+			notify_results(ctx);
+		}
+		ctx->getaddrinfo_cancelled = TRUE;
+#else
 		notify_results(ctx);
+#endif
 		return BELLE_SIP_STOP;
 	}
 	if (error != DNS_EAGAIN) {
 		belle_sip_error("%s dns_res_check() error: %s (%d)", __FUNCTION__, dns_strerror(error), error);
+#ifdef USE_GETADDRINFO_FALLBACK
+		if (ctx->getaddrinfo_done) {
+			return BELLE_SIP_STOP;
+		} else {
+			// Wait for the getaddrinfo result
+			return BELLE_SIP_CONTINUE;
+		}
+#else
 		notify_results(ctx);
 		return BELLE_SIP_STOP;
-	}else{
-		belle_sip_message("%s dns_res_check() in progress",__FUNCTION__);
+#endif
+	} else {
+#ifdef USE_GETADDRINFO_FALLBACK
+		if (ctx->getaddrinfo_done) {
+			return BELLE_SIP_STOP;
+		} else
+#endif
+		{
+			belle_sip_message("%s dns_res_check() in progress", __FUNCTION__);
+		}
 	}
 	return BELLE_SIP_CONTINUE;
 }
 
+#ifdef USE_GETADDRINFO_FALLBACK
+static void * _resolver_getaddrinfo_thread(void *ptr) {
+	belle_sip_simple_resolver_context_t *ctx = (belle_sip_simple_resolver_context_t *)ptr;
+	struct addrinfo *res = NULL;
+	struct addrinfo hints = { 0 };
+	char serv[10];
+	int err;
+
+	belle_sip_message("Resolver getaddrinfo thread started.");
+	snprintf(serv, sizeof(serv), "%i", ctx->port);
+	hints.ai_family = ctx->family;
+	hints.ai_flags = AI_NUMERICSERV;
+	err = getaddrinfo(ctx->name, serv, &hints, &res);
+	if (err != 0) {
+		belle_sip_error("getaddrinfo DNS resolution of %s failed: %s", ctx->name, gai_strerror(err));
+	} else if (!ctx->getaddrinfo_cancelled) {
+		append_dns_result(ctx, res->ai_addr, res->ai_addrlen);
+	}
+	if (res) freeaddrinfo(res);
+	ctx->getaddrinfo_done = TRUE;
+#ifdef WIN32
+	SetEvent(ctx->ctlevent);
+#else
+	if (write(ctx->ctlpipe[1], "q", 1) == -1) {
+		belle_sip_error("_resolver_getaddrinfo_thread(): Fail to write on pipe.");
+	}
+#endif
+	return NULL;
+}
+
+static int _resolver_getaddrinfo_callback(belle_sip_simple_resolver_context_t *ctx, unsigned int revents) {
+	if (!ctx->getaddrinfo_cancelled && !ctx->getaddrinfo_notified) {
+		notify_results(ctx);
+		ctx->getaddrinfo_notified = TRUE;
+	}
+	belle_sip_object_unref(ctx);
+	return BELLE_SIP_STOP;
+}
+
+static void _resolver_getaddrinfo_start(belle_sip_simple_resolver_context_t *ctx) {
+	belle_sip_fd_t fd = (belle_sip_fd_t)-1;
+
+	belle_sip_object_ref(ctx);
+#ifdef WIN32
+	ctx->ctlevent = CreateEventEx(NULL, NULL, CREATE_EVENT_MANUAL_RESET, EVENT_ALL_ACCESS);
+	fd = (HANDLE)ctx->ctlevent;
+#else
+	if (pipe(ctx->ctlpipe) == -1) {
+		belle_sip_fatal("pipe() failed: %s", strerror(errno));
+	}
+	fd = ctx->ctlpipe[0];
+#endif
+	belle_sip_thread_create(&ctx->getaddrinfo_thread, NULL, _resolver_getaddrinfo_thread, ctx);
+	ctx->getaddrinfo_source = belle_sip_fd_source_new((belle_sip_source_func_t)_resolver_getaddrinfo_callback, ctx, fd, BELLE_SIP_EVENT_READ, -1);
+	belle_sip_main_loop_add_source(ctx->base.stack->ml, ctx->getaddrinfo_source);
+}
+#endif
+
 static int _resolver_send_query(belle_sip_simple_resolver_context_t *ctx) {
-	int error;
+	int error = 0;
 
 	if (!ctx->base.stack->resolver_send_error) {
 		error = dns_res_submit(ctx->R, ctx->name, ctx->type, DNS_C_IN);
@@ -498,6 +604,11 @@ static int _resolver_send_query(belle_sip_simple_resolver_context_t *ctx) {
 		/*only init source if res inprogress*/
 		/*the timeout set to the source is 1 s, this is to allow dns.c to send request retransmissions*/
 		belle_sip_socket_source_init((belle_sip_source_t*)ctx, (belle_sip_source_func_t)resolver_process_data, ctx, dns_res_pollfd(ctx->R), BELLE_SIP_EVENT_READ | BELLE_SIP_EVENT_TIMEOUT, 1000);
+#ifdef USE_GETADDRINFO_FALLBACK
+		if ((ctx->type == DNS_T_A) || (ctx->type == DNS_T_AAAA)) {
+			_resolver_getaddrinfo_start(ctx);
+		}
+#endif
 	}
 	return 0;
 }
@@ -511,10 +622,7 @@ static int resolver_process_data_delayed(belle_sip_simple_resolver_context_t *ct
 
 static int _resolver_start_query(belle_sip_simple_resolver_context_t *ctx) {
 	struct dns_hints *(*hints)() = &dns_hints_local;
-	struct dns_options *opts;
-#ifndef HAVE_C99
-	struct dns_options opts_st;
-#endif
+	struct dns_options opts;
 	int error;
 	struct dns_resolv_conf *conf;
 
@@ -530,10 +638,9 @@ static int _resolver_start_query(belle_sip_simple_resolver_context_t *ctx) {
 	if (!hosts(ctx))
 		return -1;
 
-	memset(&opts_st, 0, sizeof opts_st);
-	opts = &opts_st;
+	memset(&opts, 0, sizeof opts);
 
-	if (!(ctx->R = dns_res_open(ctx->resconf, ctx->hosts, dns_hints_mortal(hints(ctx->resconf, &error)), cache(ctx), opts, &error))) {
+	if (!(ctx->R = dns_res_open(ctx->resconf, ctx->hosts, dns_hints_mortal(hints(ctx->resconf, &error)), cache(ctx), &opts, &error))) {
 		belle_sip_error("%s dns_res_open error [%s]: %s", __FUNCTION__, ctx->name, dns_strerror(error));
 		return -1;
 	}
@@ -619,6 +726,20 @@ static void belle_sip_combined_resolver_context_destroy(belle_sip_combined_resol
 static void belle_sip_simple_resolver_context_destroy(belle_sip_simple_resolver_context_t *ctx){
 	/* Do not free elements of ctx->ai_list with belle_sip_freeaddrinfo(). Let the caller do it, otherwise
 	   it will not be able to use them after the resolver has been destroyed. */
+#ifdef USE_GETADDRINFO_FALLBACK
+	if (ctx->getaddrinfo_thread != 0) {
+		belle_sip_thread_join(ctx->getaddrinfo_thread, NULL);
+	}
+	belle_sip_mutex_destroy(&ctx->getaddrinfo_mutex);
+#ifdef WIN32
+	if (ctx->ctlevent != (belle_sip_fd_t)-1)
+		CloseHandle(ctx->ctlevent);
+#else
+	close(ctx->ctlpipe[0]);
+	close(ctx->ctlpipe[1]);
+#endif
+
+#endif
 	if (ctx->name != NULL) {
 		belle_sip_free(ctx->name);
 		ctx->name = NULL;
@@ -891,6 +1012,12 @@ static belle_sip_resolver_context_t * belle_sip_stack_resolve_single(belle_sip_s
 	if (family == 0) family = AF_UNSPEC;
 	ctx->family = family;
 	ctx->type = (ctx->family == AF_INET6) ? DNS_T_AAAA : DNS_T_A;
+#ifdef USE_GETADDRINFO_FALLBACK
+	belle_sip_mutex_init(&ctx->getaddrinfo_mutex, NULL);
+#ifdef WIN32
+	ctx->ctlevent = (belle_sip_fd_t)-1;
+#endif
+#endif
 	return (belle_sip_resolver_context_t*)resolver_start_query(ctx);
 }
 
