@@ -95,7 +95,7 @@ belle_sip_error_code belle_sip_uri_marshal(const belle_sip_uri_t* uri, char* buf
 	error=belle_sip_snprintf(buff,buff_size,offset,"%s:",uri->secure?"sips":"sip");
 	if (error!=BELLE_SIP_OK) return error;
 	
-	if (uri->user) {
+	if (uri->user && uri->user[0]!='\0') {
 		char* escaped_username=belle_sip_uri_to_escaped_username(uri->user);
 		error=belle_sip_snprintf(buff,buff_size,offset,"%s",escaped_username);
 		belle_sip_free(escaped_username);
@@ -123,7 +123,7 @@ belle_sip_error_code belle_sip_uri_marshal(const belle_sip_uri_t* uri, char* buf
 		belle_sip_warning("no host found in this uri");
 	}
 	
-	if (uri->port>0) {
+	if (uri->port!=0) {
 		error=belle_sip_snprintf(buff,buff_size,offset,":%i",uri->port);
 		if (error!=BELLE_SIP_OK) return error;
 	}
@@ -179,7 +179,7 @@ belle_sip_uri_t* belle_sip_uri_create (const char* username,const char* host) {
 }
 
 
-char* belle_sip_uri_to_string(belle_sip_uri_t* uri)  {
+char* belle_sip_uri_to_string(const belle_sip_uri_t* uri)  {
 	return belle_sip_object_to_string(BELLE_SIP_OBJECT(uri));
 }
 
