@@ -600,7 +600,10 @@ int belle_sip_channel_process_data(belle_sip_channel_t *obj,unsigned int revents
 	}
 	if (revents & BELLE_SIP_EVENT_WRITE){
 		/*if we are here, this is because we had an EWOULDBLOCK while sending a message*/
-		channel_process_queue(obj);
+		/*continue to send pending messages but before check the channel is still alive because it may have been closed by belle_sip_channel_process_read_data() above.*/
+		if (obj->state == BELLE_SIP_CHANNEL_READY){
+			channel_process_queue(obj);
+		}
 	}
 	return ret;
 }
@@ -668,7 +671,7 @@ void belle_sip_channel_init(belle_sip_channel_t *obj, belle_sip_stack_t *stack,c
 }
 
 /*constructor for channels created by incoming connections*/
-void belle_sip_channel_init_with_addr(belle_sip_channel_t *obj, belle_sip_stack_t *stack, const struct sockaddr *peer_addr, socklen_t addrlen){
+void belle_sip_channel_init_with_addr(belle_sip_channel_t *obj, belle_sip_stack_t *stack, const char *bindip, int localport, const struct sockaddr *peer_addr, socklen_t addrlen){
 	char remoteip[64];
 	struct addrinfo ai;
 	int peer_port;
@@ -678,7 +681,7 @@ void belle_sip_channel_init_with_addr(belle_sip_channel_t *obj, belle_sip_stack_
 	ai.ai_addr=(struct sockaddr*)peer_addr;
 	ai.ai_addrlen=addrlen;
 	belle_sip_addrinfo_to_ip(&ai,remoteip,sizeof(remoteip),&peer_port);
-	belle_sip_channel_init(obj,stack,NULL,0,NULL,remoteip,peer_port);
+	belle_sip_channel_init(obj,stack,bindip,localport,NULL,remoteip,peer_port);
 	obj->peer_list=obj->current_peer=belle_sip_ip_address_to_addrinfo(ai.ai_family, obj->peer_name,obj->peer_port);
 	obj->ai_family=ai.ai_family;
 }
@@ -907,10 +910,15 @@ static void handle_ewouldblock(belle_sip_channel_t *obj, const char *buffer, siz
 	memcpy(obj->ewouldblock_buffer,buffer,size);
 }
 
-static size_t find_non_asci(const char *buffer, size_t size){
+static size_t find_non_printable(const char *buffer, size_t size){
 	size_t i;
 	for(i=0;i<size;++i){
-		if (!isascii(buffer[i])) return i;
+		/*we must check that character is printable, not just ascii
+		(31 'US' char is not printable for instance)*/
+#ifdef _MSC_VER
+		if ((buffer[i] < 0) || (buffer[i] > 255)) return i;
+#endif
+		if (!isprint(buffer[i]) && !isspace(buffer[i])) return i;
 	}
 	return size;
 }
@@ -921,25 +929,22 @@ static char *make_logbuf(belle_sip_log_level level, const char *buffer, size_t s
 	char *logbuf;
 	char truncate_msg[128]={0};
 	int limit=7000; /*big message when many ice candidates*/
-	size_t non_ascii_pos;
 
 	if (!belle_sip_log_level_enabled(level)){
 		return belle_sip_malloc0(1);
 	}
-	non_ascii_pos=find_non_asci(buffer,MIN(size,limit));
-	if (non_ascii_pos<2){
-		limit=0;
+	limit=find_non_printable(buffer,MIN(size,limit));
+	if (limit==0){
 		snprintf(truncate_msg,sizeof(truncate_msg)-1,"... (binary data)");
-	}else limit=non_ascii_pos;
-	if (limit!=0 && size>limit){
-		snprintf(truncate_msg,sizeof(truncate_msg)-1," ... (first %i bytes shown)",limit);
+	} else if (size>limit){
+		snprintf(truncate_msg,sizeof(truncate_msg)-1,"... (first %i bytes shown)",limit);
 		size=limit;
 	}
-	if (size<limit)
-		limit=size;
+
 	if (truncate_msg[0]!=0){
 		size+=sizeof(truncate_msg);
 	}
+
 	logbuf=belle_sip_malloc(size+1);
 	strncpy(logbuf,buffer,limit);
 	if (truncate_msg[0]!=0){
@@ -964,7 +969,7 @@ static int send_buffer(belle_sip_channel_t *obj, const char *buffer, size_t size
 
 	if (ret<0){
 		if (!belle_sip_error_code_is_would_block(-ret)){
-			belle_sip_error("channel [%p]: could not send [%i] bytes from [%s://%s:%i]  to [%s:%i]"	,obj
+			belle_sip_error("channel [%p]: could not send [%i] bytes from [%s://%s:%i] to [%s:%i]"	,obj
 				,(int)size
 				,belle_sip_channel_get_transport_name(obj)
 				,obj->local_ip
