@@ -93,8 +93,8 @@ static void belle_sip_headers_container_delete(headers_container_t *obj){
 
 static void belle_sip_message_destroy(belle_sip_message_t *msg){
 	belle_sip_list_free_with_data(msg->header_list,(void (*)(void*))belle_sip_headers_container_delete);
-	if (msg->body)
-		belle_sip_free(msg->body);
+	if (msg->body_handler)
+		belle_sip_object_unref(msg->body_handler);
 }
 
 /*very sub-optimal clone method */
@@ -244,8 +244,8 @@ void belle_sip_message_remove_last(belle_sip_message_t *msg, const char *header_
 void belle_sip_message_remove_header(belle_sip_message_t *msg, const char *header_name){
 	headers_container_t* headers_container = belle_sip_headers_container_get(msg,header_name);
 	if (headers_container){
-		belle_sip_headers_container_delete(headers_container);
 		belle_sip_list_remove(msg->header_list,headers_container);
+		belle_sip_headers_container_delete(headers_container);
 	}
 }
 
@@ -305,17 +305,36 @@ belle_sip_error_code belle_sip_headers_marshal(belle_sip_message_t *message, cha
 	belle_sip_list_t* headers_list;
 	belle_sip_list_t* header_list;
 	belle_sip_error_code error=BELLE_SIP_OK;
+#ifdef BELLE_SIP_WORKAROUND_TECHNICOLOR_SIP_ALG_ROUTER_BUG
+	belle_sip_header_t *content_length=NULL;
+#endif
+	
 	for(headers_list=message->header_list;headers_list!=NULL;headers_list=headers_list->next){
 		for(header_list=((headers_container_t*)(headers_list->data))->header_list
 				;header_list!=NULL
 				;header_list=header_list->next)	{
 			belle_sip_header_t *h=BELLE_SIP_HEADER(header_list->data);
-			error=belle_sip_object_marshal(BELLE_SIP_OBJECT(h),buff,buff_size,offset);
-			if (error!=BELLE_SIP_OK) return error;
-			error=belle_sip_snprintf(buff,buff_size,offset,"%s","\r\n");
-			if (error!=BELLE_SIP_OK) return error;
+#ifdef BELLE_SIP_WORKAROUND_TECHNICOLOR_SIP_ALG_ROUTER_BUG
+			if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(h,belle_sip_header_content_length_t)){
+				content_length=h;
+			}else
+#endif
+			{
+				error=belle_sip_object_marshal(BELLE_SIP_OBJECT(h),buff,buff_size,offset);
+				if (error!=BELLE_SIP_OK) return error;
+				error=belle_sip_snprintf(buff,buff_size,offset,"%s","\r\n");
+				if (error!=BELLE_SIP_OK) return error;
+			}
 		}
 	}
+#ifdef BELLE_SIP_WORKAROUND_TECHNICOLOR_SIP_ALG_ROUTER_BUG
+	if (content_length){
+		error=belle_sip_object_marshal(BELLE_SIP_OBJECT(content_length),buff,buff_size,offset);
+		if (error!=BELLE_SIP_OK) return error;
+		error=belle_sip_snprintf(buff,buff_size,offset,"%s","\r\n");
+		if (error!=BELLE_SIP_OK) return error;
+	}
+#endif
 	error=belle_sip_snprintf(buff,buff_size,offset,"%s","\r\n");
 	if (error!=BELLE_SIP_OK) return error;
 	return error;
@@ -324,6 +343,7 @@ belle_sip_error_code belle_sip_headers_marshal(belle_sip_message_t *message, cha
 static void belle_sip_request_destroy(belle_sip_request_t* request) {
 	if (request->method) belle_sip_free(request->method);
 	if (request->uri) belle_sip_object_unref(request->uri);
+	if (request->absolute_uri) belle_sip_object_unref(request->absolute_uri);
 	if (request->dialog) belle_sip_object_unref(request->dialog);
 	if (request->rfc2543_branch) belle_sip_free(request->rfc2543_branch);
 }
@@ -334,22 +354,30 @@ static void belle_sip_request_init(belle_sip_request_t *message){
 static void belle_sip_request_clone(belle_sip_request_t *request, const belle_sip_request_t *orig){
 	if (orig->method) request->method=belle_sip_strdup(orig->method);
 	if (orig->uri) request->uri=(belle_sip_uri_t*)belle_sip_object_ref(belle_sip_object_clone((belle_sip_object_t*)orig->uri));
+	if (orig->absolute_uri) request->absolute_uri=(belle_generic_uri_t*)belle_sip_object_ref(belle_sip_object_clone((belle_sip_object_t*)orig->absolute_uri));
 	if (orig->rfc2543_branch) request->rfc2543_branch=belle_sip_strdup(orig->rfc2543_branch);
 }
 
 belle_sip_error_code belle_sip_request_marshal(belle_sip_request_t* request, char* buff, size_t buff_size, size_t *offset) {
 	belle_sip_error_code error=belle_sip_snprintf(buff,buff_size,offset,"%s ",belle_sip_request_get_method(request));
+
 	if (error!=BELLE_SIP_OK) return error;
-	error=belle_sip_uri_marshal(belle_sip_request_get_uri(request),buff,buff_size,offset);
+	if (request->uri)
+		error=belle_sip_uri_marshal(belle_sip_request_get_uri(request),buff,buff_size,offset);
+	else if (request->absolute_uri)
+		error=belle_generic_uri_marshal(belle_sip_request_get_absolute_uri(request),buff,buff_size,offset);
+	else {
+		belle_sip_error("Missing uri for marshaling request [%p]",request);
+		/*fixme better to have an error code*/
+		error=BELLE_SIP_OK;
+	}
+
 	if (error!=BELLE_SIP_OK) return error;
 	error=belle_sip_snprintf(buff,buff_size,offset," %s","SIP/2.0\r\n");
 	if (error!=BELLE_SIP_OK) return error;
 	error=belle_sip_headers_marshal(BELLE_SIP_MESSAGE(request),buff,buff_size,offset);
 	if (error!=BELLE_SIP_OK) return error;
-	if (BELLE_SIP_MESSAGE(request)->body) {
-		error=belle_sip_snprintf(buff,buff_size,offset, "%s",BELLE_SIP_MESSAGE(request)->body);
-		if (error!=BELLE_SIP_OK) return error;
-	}
+	
 	return error;
 }
 
@@ -358,26 +386,36 @@ BELLE_SIP_PARSE(request)
 GET_SET_STRING(belle_sip_request,method);
 GET_SET_STRING(belle_sip_request,rfc2543_branch);
 
+/*caching of the dialog in the request, used when creating a request in dialog to avoid dialog lookup*/
 void belle_sip_request_set_dialog(belle_sip_request_t *req, belle_sip_dialog_t *dialog){
-	if (dialog) belle_sip_object_ref(dialog);
-	if (req->dialog) {
-		belle_sip_object_unref(req->dialog);
-	}
-	req->dialog=dialog;
+	SET_OBJECT_PROPERTY(req,dialog,dialog);
 }
 
 void belle_sip_request_set_uri(belle_sip_request_t* request,belle_sip_uri_t* uri) {
-	belle_sip_object_ref(uri);
-	if (request->uri) {
-		belle_sip_object_unref(request->uri);
+	SET_OBJECT_PROPERTY(request,uri,uri);
+	if (request->absolute_uri && uri) {
+		belle_sip_warning("absolute uri [%p] already set for request [%p], cleaning it",request->absolute_uri, request);
+		belle_sip_request_set_absolute_uri(request,NULL);
 	}
-	request->uri=uri;
 }
 
 belle_sip_uri_t * belle_sip_request_get_uri(const belle_sip_request_t *request){
 	return request->uri;
 }
 
+void belle_sip_request_set_absolute_uri(belle_sip_request_t* request,belle_generic_uri_t* absolute_uri) {
+	SET_OBJECT_PROPERTY(request,absolute_uri,absolute_uri);
+	if (request->uri && absolute_uri) {
+		belle_sip_warning("sip  uri [%p] already set for request [%p], cleaning it",request->uri, request);
+		belle_sip_request_set_uri(request,NULL);
+	}
+}
+
+belle_generic_uri_t * belle_sip_request_get_absolute_uri(const belle_sip_request_t *request){
+	return request->absolute_uri;
+}
+
+
 belle_sip_uri_t* belle_sip_request_extract_origin(const belle_sip_request_t* req) {
 	belle_sip_header_via_t* via_header = belle_sip_message_get_header_by_type(req,belle_sip_header_via_t);
 	belle_sip_uri_t* uri=NULL;
@@ -419,33 +457,48 @@ char *belle_sip_message_to_string(belle_sip_message_t *msg){
 	return belle_sip_object_to_string(BELLE_SIP_OBJECT(msg));
 }
 
-const char* belle_sip_message_get_body(belle_sip_message_t *msg) {
-	return msg->body;
+belle_sip_body_handler_t *belle_sip_message_get_body_handler(const belle_sip_message_t *msg){
+	return msg->body_handler;
 }
 
-unsigned int belle_sip_message_get_body_size(const belle_sip_message_t *msg){
-	return msg->body_length;
+void belle_sip_message_set_body_handler(belle_sip_message_t *msg, belle_sip_body_handler_t *body_handler){
+	/* In case of multipart message, we must add the message Content-Type header containing the boundary */
+	if (body_handler != NULL) {
+		if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(body_handler, belle_sip_multipart_body_handler_t)){
+			char *content_type = belle_sip_strdup_printf("multipart/form-data; boundary=%s",BELLESIP_MULTIPART_BOUNDARY);
+			belle_sip_message_add_header(BELLE_SIP_MESSAGE(msg), belle_sip_header_create("Content-type",content_type));
+			belle_sip_free(content_type);
+		}
+	}
+
+	SET_OBJECT_PROPERTY(msg,body_handler,body_handler);
+}
+
+const char* belle_sip_message_get_body(belle_sip_message_t *msg) {
+	if (msg->body_handler==NULL) return NULL;
+	if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(msg->body_handler, belle_sip_memory_body_handler_t)){
+		return (const char*)belle_sip_memory_body_handler_get_buffer(
+			BELLE_SIP_MEMORY_BODY_HANDLER(msg->body_handler)
+		);
+	}
+	belle_sip_error("belle_sip_message_get_body(): body cannot be returned as pointer.");
+	return NULL;
+}
+
+size_t belle_sip_message_get_body_size(const belle_sip_message_t *msg){
+	if (msg->body_handler==NULL) return 0;
+	return belle_sip_body_handler_get_size(msg->body_handler);
 }
 
 void belle_sip_message_set_body(belle_sip_message_t *msg, const char* body, unsigned int size) {
-	if (msg->body) {
-		belle_sip_free(msg->body);
-		msg->body=NULL;
-	}
-	if (body){
-		msg->body = belle_sip_malloc(size+1);
-		msg->body_length=size;
-		memcpy(msg->body,body,size);
-		msg->body[size]='\0';
-	}
+	belle_sip_body_handler_t *bh=NULL;
+	if (body && size) bh=(belle_sip_body_handler_t*)belle_sip_memory_body_handler_new_copy_from_buffer(body,size,NULL,NULL);
+	belle_sip_message_set_body_handler(msg,bh);
 }
 
 void belle_sip_message_assign_body(belle_sip_message_t *msg, char* body, unsigned int size) {
-	if (msg->body) {
-		belle_sip_free(body);
-	}
-	msg->body = body;
-	msg->body_length = size;
+	belle_sip_body_handler_t *bh=(belle_sip_body_handler_t*)belle_sip_memory_body_handler_new_from_buffer(body,size,NULL,NULL);
+	belle_sip_message_set_body_handler(msg,bh);
 }
 
 struct _belle_sip_response{
@@ -484,6 +537,7 @@ static code_phrase_t well_known_codes[]={
 	{	407		,		"Proxy authentication required"	},
 	{	408		,		"Request timeout"	},
 	{	410		,		"Gone"	},
+	{	412		,		"Conditional Request Failed" }, /*rfc3903*/
 	{	413		,		"Request entity too large"	},
 	{	414		,		"Request-URI too long"	},
 	{	415		,		"Unsupported media type"	},
@@ -500,6 +554,7 @@ static code_phrase_t well_known_codes[]={
 	{	486		,		"Busy here"	},
 	{	487		,		"Request terminated"	},
 	{	488		,		"Not acceptable here"	},
+	{	489		,		"Bad Event"	}, /*rfc3265*/
 	{	491		,		"Request pending"	},
 	{	493		,		"Undecipherable"	},
 	{	500		,		"Server internal error"	},
@@ -540,18 +595,15 @@ static void belle_sip_response_clone(belle_sip_response_t *resp, const belle_sip
 
 belle_sip_error_code belle_sip_response_marshal(belle_sip_response_t *resp, char* buff, size_t buff_size, size_t *offset) {
 	belle_sip_error_code error=belle_sip_snprintf(	buff
-													,buff_size
-													,offset
-													,"SIP/2.0 %i %s\r\n"
-													,belle_sip_response_get_status_code(resp)
-													,belle_sip_response_get_reason_phrase(resp)?belle_sip_response_get_reason_phrase(resp):"");
+						,buff_size
+						,offset
+						,"SIP/2.0 %i %s\r\n"
+						,belle_sip_response_get_status_code(resp)
+						,belle_sip_response_get_reason_phrase(resp)?belle_sip_response_get_reason_phrase(resp):"");
+	
 	if (error!=BELLE_SIP_OK) return error;
 	error=belle_sip_headers_marshal(BELLE_SIP_MESSAGE(resp),buff,buff_size,offset);
 	if (error!=BELLE_SIP_OK) return error;
-	if (BELLE_SIP_MESSAGE(resp)->body) {
-		error=belle_sip_snprintf(buff,buff_size,offset, "%s",BELLE_SIP_MESSAGE(resp)->body);
-		if (error!=BELLE_SIP_OK) return error;
-	}
 	return error;
 }
 
@@ -781,15 +833,8 @@ int belle_sip_response_fix_contact(const belle_sip_response_t* response,belle_si
 
 belle_sip_request_t * belle_sip_request_clone_with_body(const belle_sip_request_t *initial_req) {
 	belle_sip_request_t* req=BELLE_SIP_REQUEST(belle_sip_object_clone(BELLE_SIP_OBJECT(initial_req)));
-	if (belle_sip_message_get_body(BELLE_SIP_MESSAGE(initial_req))) {
-		belle_sip_header_content_length_t* content_lenth=belle_sip_message_get_header_by_type(initial_req,belle_sip_header_content_length_t);
-		if (content_lenth)
-			belle_sip_message_set_body(BELLE_SIP_MESSAGE(req)
-										,belle_sip_message_get_body(BELLE_SIP_MESSAGE(initial_req))
-										,belle_sip_header_content_length_get_content_length(content_lenth));
-		else
-			belle_sip_error("Cannot clone body from request [%p] because no content lenght header",initial_req);
-	}
+	if (initial_req->base.body_handler) req->base.body_handler=BELLE_SIP_BODY_HANDLER(belle_sip_object_clone_and_ref(
+		(belle_sip_object_t*)initial_req->base.body_handler));
 	return req;
 }
 
@@ -878,6 +923,7 @@ static message_header_list_t mandatory_headers[] = {
 		{"CANCEL",{"Call-ID","CSeq","From", "Max-Forwards","To","Via",NULL}},
 		{"BYE",{"Call-ID","CSeq","From", "Max-Forwards","To","Via",NULL}},
 		{"ACK",{"Call-ID","CSeq","From", "Max-Forwards","To","Via",NULL}},
+		{"*", { "To", "From", "CSeq", "Via", NULL}}, /* catch-all, these fields are required all the time. */
 		{NULL,{NULL}}
 };
 
@@ -903,11 +949,12 @@ int belle_sip_message_check_headers(const belle_sip_message_t* message) {
 		const char * method = belle_sip_request_get_method(BELLE_SIP_REQUEST(message));
 		
 		for (i=0;mandatory_headers[i].method!=NULL;i++) {
-			if (strcasecmp(method,mandatory_headers[i].method)==0){
+			if ( (strcasecmp(method,mandatory_headers[i].method)==0) ||
+				 (mandatory_headers[i].method[0] == '*') ){
 				int j;
 				for(j=0;mandatory_headers[i].headers[j]!=NULL;j++) {
 					if (belle_sip_message_get_header(message,mandatory_headers[i].headers[j])==NULL) {
-						belle_sip_error("Missing mandatory header [%s] for message [%s]",mandatory_headers[i].headers[j],mandatory_headers[i].method);
+						belle_sip_error("Missing mandatory header [%s] for message [%s]",mandatory_headers[i].headers[j],method);
 						return 0;
 					}
 				}
