@@ -22,6 +22,8 @@
 #define DEFAULT_RETRY_AFTER 60000
 #define DEFAULT_INITIAL_RETRY_AFTER_ON_IO_ERROR 500
 
+static void belle_sip_refresher_stop_internal(belle_sip_refresher_t* refresher,int cancel_pending_transaction) ;
+
 typedef enum belle_sip_refresher_state {
 	started,
 	stopped
@@ -37,6 +39,7 @@ struct belle_sip_refresher {
 	belle_sip_refresher_listener_t listener;
 	belle_sip_source_t* timer;
 	belle_sip_client_transaction_t* transaction;
+	char* realm;
 	int target_expires;
 	int obtained_expires;
 	belle_sip_refresher_state_t state;
@@ -54,14 +57,18 @@ static int set_expires_from_trans(belle_sip_refresher_t* refresher);
 static int timer_cb(void *user_data, unsigned int events) ;
 static int belle_sip_refresher_refresh_internal(belle_sip_refresher_t* refresher,int expires,int auth_mandatory, belle_sip_list_t** auth_infos, belle_sip_uri_t *requri);
 
-
-static void schedule_timer_at(belle_sip_refresher_t* refresher,int delay, timer_purpose_t purpose) {
-	belle_sip_message("Refresher: scheduling next timer in %i ms",delay);
-	refresher->timer_purpose=purpose;
+static void cancel_retry(belle_sip_refresher_t* refresher) {
 	if (refresher->timer){
 		belle_sip_main_loop_remove_source(belle_sip_stack_get_main_loop(refresher->transaction->base.provider->stack),refresher->timer);
 		belle_sip_object_unref(refresher->timer);
+		refresher->timer=NULL;
 	}
+}
+static void schedule_timer_at(belle_sip_refresher_t* refresher,int delay, timer_purpose_t purpose) {
+	belle_sip_message("Refresher: scheduling next timer in %i ms",delay);
+	refresher->timer_purpose=purpose;
+	/*cancel timer if any*/
+	cancel_retry(refresher);
 	refresher->timer=belle_sip_timeout_source_new(timer_cb,refresher,delay);
 	belle_sip_object_set_name((belle_sip_object_t*)refresher->timer,"Refresher timeout");
 	belle_sip_main_loop_add_source(belle_sip_stack_get_main_loop(refresher->transaction->base.provider->stack),refresher->timer);
@@ -187,9 +194,8 @@ static void process_response_event(belle_sip_listener_t *user_ctx, const belle_s
 	if (refresher && (client_transaction !=refresher->transaction))
 		return; /*not for me*/
 
-	/*handle authorization*/
-	switch (response_code) {
-	case 200:
+	/*success case:*/
+	if (response_code>=200 && response_code<300){
 		refresher->auth_failures=0;
 		refresher->number_of_retry=0;
 		/*great, success*/
@@ -220,58 +226,61 @@ static void process_response_event(belle_sip_listener_t *user_ctx, const belle_s
 			}
 		}
 		else belle_sip_message("Refresher [%p] not scheduling next refresh, because it was stopped",refresher);
-		break;
-	case 301:
-	case 302:
-		contact=belle_sip_message_get_header_by_type(response,belle_sip_header_contact_t);
-		if (contact){
-			belle_sip_uri_t *uri=belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(contact));
-			if (uri && belle_sip_refresher_refresh_internal(refresher,refresher->target_expires,TRUE,&refresher->auth_events,uri)==0)
-				return;
+	}else{/*special error cases*/
+		switch (response_code) {
+		case 301:
+		case 302:
+			contact=belle_sip_message_get_header_by_type(response,belle_sip_header_contact_t);
+			if (contact){
+				belle_sip_uri_t *uri=belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(contact));
+				if (uri && belle_sip_refresher_refresh_internal(refresher,refresher->target_expires,TRUE,&refresher->auth_events,uri)==0)
+					return;
+			}
+			break;
+		case 401:
+		case 407:
+			refresher->auth_failures++;
+			if (refresher->auth_failures>1){
+				/*avoid looping with 407 or 401 */
+				belle_sip_warning("Authentication is failing constantly, %s",(refresher->target_expires>0)? "will retry later":"giving up.");
+				if (refresher->target_expires>0) retry_later(refresher);
+				refresher->auth_failures=0; /*reset auth failure*/
+				break;
+			}
+			if (refresher->auth_events) {
+				refresher->auth_events=belle_sip_list_free_with_data(refresher->auth_events,(void (*)(void*))belle_sip_auth_event_destroy);
+			}
+			if (belle_sip_refresher_refresh_internal(refresher,refresher->target_expires,TRUE,&refresher->auth_events,NULL)==0)
+				return; /*ok, keep 401 internal*/
+			break; /*Else notify user of registration failure*/
+		case 403:
+			/*In case of 403, we will retry later, just in case*/
+			if (refresher->target_expires>0) retry_later(refresher);
+			break;
+		case 423:{
+			belle_sip_header_extension_t *min_expires=BELLE_SIP_HEADER_EXTENSION(belle_sip_message_get_header((belle_sip_message_t*)response,"Min-Expires"));
+			if (min_expires){
+				const char *value=belle_sip_header_extension_get_value(min_expires);
+				if (value){
+					int new_expires=atoi(value);
+					if (new_expires>0 && refresher->state==started){
+						refresher->target_expires=new_expires;
+						belle_sip_refresher_refresh(refresher,refresher->target_expires);
+						return;
+					}
+				}
+			}else belle_sip_warning("Receiving 423 but no min-expires header.");
+			break;
 		}
-		break;
-	case 401:
-	case 407:
-		refresher->auth_failures++;
-		if (refresher->auth_failures>1){
-			/*avoid looping with 407 or 401 */
-			belle_sip_warning("Authentication is failing constantly, %s",(refresher->target_expires>0)? "will retry later":"giving up.");
+		case 505:
+		case 501:
+			/*irrecoverable errors, probably no need to retry later*/
+			break;
+		default:
+			/*for all other errors, retry later*/
 			if (refresher->target_expires>0) retry_later(refresher);
 			break;
 		}
-		if (refresher->auth_events) {
-			refresher->auth_events=belle_sip_list_free_with_data(refresher->auth_events,(void (*)(void*))belle_sip_auth_event_destroy);
-		}
-		if (belle_sip_refresher_refresh_internal(refresher,refresher->target_expires,TRUE,&refresher->auth_events,NULL)==0)
-			return; /*ok, keep 401 internal*/
-		break; /*Else notify user of registration failure*/
-	case 403:
-		/*In case of 403, we will retry later, just in case*/
-		if (refresher->target_expires>0) retry_later(refresher);
-		break;
-	case 423:{
-		belle_sip_header_extension_t *min_expires=BELLE_SIP_HEADER_EXTENSION(belle_sip_message_get_header((belle_sip_message_t*)response,"Min-Expires"));
-		if (min_expires){
-			const char *value=belle_sip_header_extension_get_value(min_expires);
-			if (value){
-				int new_expires=atoi(value);
-				if (new_expires>0 && refresher->state==started){
-					refresher->target_expires=new_expires;
-					belle_sip_refresher_refresh(refresher,refresher->target_expires);
-					return;
-				}
-			}
-		}else belle_sip_warning("Receiving 423 but no min-expires header.");
-		break;
-	}
-	case 408:
-	case 480:
-	case 503:
-	case 504:
-		if (refresher->target_expires>0) retry_later(refresher);
-		break;
-	default:
-		break;
 	}
 	if (refresher->listener) refresher->listener(refresher,refresher->user_data,response_code, belle_sip_response_get_reason_phrase(response));
 
@@ -306,6 +315,7 @@ static void destroy(belle_sip_refresher_t *refresher){
 	belle_sip_provider_remove_internal_sip_listener(refresher->transaction->base.provider,BELLE_SIP_LISTENER(refresher));
 	belle_sip_object_unref(refresher->transaction);
 	refresher->transaction=NULL;
+	if (refresher->realm) belle_sip_free(refresher->realm);
 	if (refresher->auth_events) refresher->auth_events=belle_sip_list_free_with_data(refresher->auth_events,(void (*)(void*))belle_sip_auth_event_destroy);
 }
 
@@ -329,6 +339,9 @@ void belle_sip_refresher_set_listener(belle_sip_refresher_t* refresher, belle_si
 }
 
 int belle_sip_refresher_refresh(belle_sip_refresher_t* refresher,int expires) {
+	/*first cancel any current retry*/
+	cancel_retry(refresher);
+	refresher->auth_failures=0;/*reset the auth_failures to get a chance to authenticate again*/
 	return belle_sip_refresher_refresh_internal(refresher,expires,FALSE,NULL,NULL);
 }
 
@@ -347,7 +360,7 @@ static int belle_sip_refresher_refresh_internal(belle_sip_refresher_t* refresher
 	belle_sip_uri_t* preset_route=refresher->transaction->preset_route;
 	belle_sip_provider_t* prov=refresher->transaction->base.provider;
 	belle_sip_header_contact_t* contact;
-	
+
 	/*first remove timer if any*/
 	if (expires >=0) {
 		refresher->target_expires=expires;
@@ -361,14 +374,14 @@ static int belle_sip_refresher_refresh_internal(belle_sip_refresher_t* refresher
 		if (belle_sip_transaction_state_is_transient(state)) {
 			/*operation pending, cannot update authorization headers*/
 			belle_sip_header_cseq_t* cseq;
-			belle_sip_message("Refresher [%p] already have transaction [%p] in state [%s]"	,refresher
+			belle_sip_message("Refresher [%p] already has transaction [%p] in state [%s]"	,refresher
 				,refresher->transaction
 				,belle_sip_transaction_state_to_string(state));
 			request=belle_sip_request_clone_with_body(belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(refresher->transaction)));
 			cseq=belle_sip_message_get_header_by_type(request,belle_sip_header_cseq_t);
 			belle_sip_header_cseq_set_seq_number(cseq,belle_sip_header_cseq_get_seq_number(cseq)+1);
 		} else {
-			request=belle_sip_client_transaction_create_authenticated_request(refresher->transaction,auth_infos);
+			request=belle_sip_client_transaction_create_authenticated_request(refresher->transaction,auth_infos,refresher->realm);
 		}
 		if (requri){
 			/*case where we are redirected*/
@@ -390,7 +403,7 @@ static int belle_sip_refresher_refresh_internal(belle_sip_refresher_t* refresher
 				belle_sip_message_add_header(BELLE_SIP_MESSAGE(request),BELLE_SIP_HEADER(expires_header));
 			}
 		}
-		belle_sip_provider_add_authorization(prov,request,old_response,auth_infos);
+		belle_sip_provider_add_authorization(prov,request,old_response,NULL,auth_infos,refresher->realm);
 	} else {
 		belle_sip_error("Unexpected dialog state [%s] for dialog [%p], cannot refresh [%s]"
 				,belle_sip_dialog_state_to_string(belle_sip_dialog_get_state(dialog))
@@ -404,7 +417,7 @@ static int belle_sip_refresher_refresh_internal(belle_sip_refresher_t* refresher
 		if (request) belle_sip_object_unref(request);
 		return -1;
 	}
-	
+
 	refresher->on_io_error=0; /*reset this flag*/
 
 	/*update expires in any cases*/
@@ -423,38 +436,47 @@ static int belle_sip_refresher_refresh_internal(belle_sip_refresher_t* refresher
 			belle_sip_header_date_set_time(date,&curtime);
 		}
 	}
-	
+
 	client_transaction = belle_sip_provider_create_client_transaction(prov,request);
 	client_transaction->base.is_internal=1;
 	belle_sip_transaction_set_application_data(BELLE_SIP_TRANSACTION(client_transaction),refresher);
+
+	switch (belle_sip_transaction_get_state(BELLE_SIP_TRANSACTION(refresher->transaction))) {
+	case BELLE_SIP_TRANSACTION_INIT:
+	case BELLE_SIP_TRANSACTION_CALLING:
+	case BELLE_SIP_TRANSACTION_TRYING:
+		/*very early state, we can assume nobody will answer, stop retransmiting*/
+		belle_sip_transaction_terminate(BELLE_SIP_TRANSACTION(refresher->transaction));
+		break;
+	default: /*we preserve the transaction "as is"*/
+		break;
+	}
 	/*update reference transaction for next refresh*/
 	belle_sip_object_unref(refresher->transaction);
 	refresher->transaction=client_transaction;
 	belle_sip_object_ref(refresher->transaction);
 
-	if (belle_sip_client_transaction_send_request_to(client_transaction,preset_route)) {
+	if (belle_sip_client_transaction_send_request_to(client_transaction,requri?requri:preset_route)) { /*send imediatly to requri in case of redirect*/
 		belle_sip_error("Cannot send refresh method [%s] for refresher [%p]"
 				,belle_sip_request_get_method(request)
 				,refresher);
 		return -1;
 	}
-	if (expires==0) belle_sip_refresher_stop(refresher);
+	if (expires==0) belle_sip_refresher_stop_internal(refresher,0); /*unregister transaction must be preserved*/
 	return 0;
 }
 
 
 static int timer_cb(void *user_data, unsigned int events) {
 	belle_sip_refresher_t* refresher = (belle_sip_refresher_t*)user_data;
-	
-	refresher->auth_failures=0;/*reset the auth_failures to get a chance to authenticate again*/
-	
+
 	if (refresher->timer_purpose==NORMAL_REFRESH && refresher->manual) {
 		belle_sip_message("Refresher [%p] is in manual mode, skipping refresh.",refresher);
 		/*call listener with special code 0 to indicate request is about to expire*/
 		if (refresher->listener) refresher->listener(refresher,refresher->user_data,0, "about to expire");
 		return BELLE_SIP_STOP;
 	}
-	
+
 	if (belle_sip_refresher_refresh(refresher,refresher->target_expires)==-1){
 		retry_later(refresher);
 	}
@@ -513,13 +535,13 @@ static int set_expires_from_trans(belle_sip_refresher_t* refresher) {
 	belle_sip_header_contact_t* contact_header;
 
 	refresher->obtained_expires=-1;
-	
+
 	if (strcmp("REGISTER",belle_sip_request_get_method(request))==0
 			|| expires_header /*if request has an expire header, refresher can always work*/) {
 
 		if (expires_header)
 			refresher->target_expires = belle_sip_header_expires_get_expires(expires_header);
-		
+
 		/*An "expires" parameter on the "Contact" header has no semantics for
 		*   SUBSCRIBE and is explicitly not equivalent to an "Expires" header in
 		*  a SUBSCRIBE request or response.
@@ -583,14 +605,20 @@ int belle_sip_refresher_start(belle_sip_refresher_t* refresher) {
 	}
 	return 0;
 }
-
 void belle_sip_refresher_stop(belle_sip_refresher_t* refresher) {
+	belle_sip_refresher_stop_internal(refresher, 1);
+}
+
+static void belle_sip_refresher_stop_internal(belle_sip_refresher_t* refresher,int cancel_pending_transaction) {
 	belle_sip_message("Refresher [%p] stopped.",refresher);
 	if (refresher->timer){
 		belle_sip_main_loop_remove_source(belle_sip_stack_get_main_loop(refresher->transaction->base.provider->stack), refresher->timer);
 		belle_sip_object_unref(refresher->timer);
 		refresher->timer=NULL;
 	}
+	if (cancel_pending_transaction && refresher->transaction && belle_sip_transaction_state_is_transient(belle_sip_transaction_get_state(BELLE_SIP_TRANSACTION(refresher->transaction)))) {
+		belle_sip_transaction_terminate(BELLE_SIP_TRANSACTION(refresher->transaction)); /*refresher cancelled, no need to continue to retransmit*/
+	}
 	refresher->state=stopped;
 }
 
@@ -599,14 +627,14 @@ belle_sip_refresher_t* belle_sip_refresher_new(belle_sip_client_transaction_t* t
 	belle_sip_transaction_state_t state=belle_sip_transaction_get_state(BELLE_SIP_TRANSACTION(transaction));
 	belle_sip_request_t* request = belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(transaction));
 	int is_register=strcmp("REGISTER",belle_sip_request_get_method(request))==0;
-	
+
 	refresher = (belle_sip_refresher_t*)belle_sip_object_new(belle_sip_refresher_t);
 	refresher->transaction=transaction;
 	refresher->state=stopped;
 	refresher->number_of_retry=0;
 	belle_sip_object_ref(transaction);
 	refresher->retry_after=DEFAULT_RETRY_AFTER;
-	
+
 	belle_sip_provider_add_internal_sip_listener(transaction->base.provider,BELLE_SIP_LISTENER(refresher), is_register);
 	if (set_expires_from_trans(refresher)==-1){
 		belle_sip_error("Unable to extract refresh value from transaction [%p]",transaction);
@@ -632,6 +660,21 @@ int belle_sip_refresher_get_retry_after(const belle_sip_refresher_t* refresher){
 void belle_sip_refresher_set_retry_after(belle_sip_refresher_t* refresher, int delay_ms) {
 	refresher->retry_after=delay_ms;
 }
+
+const char* belle_sip_refresher_get_realm(const belle_sip_refresher_t* refresher){
+	return refresher->realm;
+}
+
+void belle_sip_refresher_set_realm(belle_sip_refresher_t* refresher, const char* realm) {
+	if (refresher->realm){
+		belle_sip_free(refresher->realm);
+		refresher->realm = NULL;
+	}
+	if (realm!=NULL){
+		refresher->realm=belle_sip_strdup(realm);
+	}
+}
+
 const belle_sip_client_transaction_t* belle_sip_refresher_get_transaction(const belle_sip_refresher_t* refresher) {
 	return refresher->transaction;
 }
@@ -643,3 +686,6 @@ void belle_sip_refresher_enable_manual_mode(belle_sip_refresher_t *refresher, in
 	refresher->manual=enabled;
 }
 
+char *belle_sip_refresher_get_public_uri(belle_sip_refresher_t* refresher)  {
+	return belle_sip_channel_get_public_ip_port(refresher->transaction->base.channel);
+}
