@@ -111,9 +111,14 @@ int belle_sip_transaction_state_is_transient(const belle_sip_transaction_state_t
 
 void belle_sip_transaction_terminate(belle_sip_transaction_t *t){
 	if (belle_sip_transaction_get_state(BELLE_SIP_TRANSACTION(t))!=BELLE_SIP_TRANSACTION_TERMINATED) {
+		int is_client=BELLE_SIP_OBJECT_IS_INSTANCE_OF(t,belle_sip_client_transaction_t);
 		belle_sip_transaction_set_state(t,BELLE_SIP_TRANSACTION_TERMINATED);
-		belle_sip_message("%s%s %s transaction [%p] terminated"	,BELLE_SIP_OBJECT_IS_INSTANCE_OF(t,belle_sip_client_transaction_t)?"Client":"Server"
-									,t->is_internal?" internal":""
+		if (t->dialog && (!t->last_response || belle_sip_response_get_status_code(t->last_response)<200)){
+			/*inform the dialog if a transaction terminates without final response.*/
+			belle_sip_dialog_update(t->dialog,t,!is_client);
+		}
+		belle_sip_message("%s%s %s transaction [%p] terminated"	,is_client ? "Client":"Server"
+									,t->is_internal ? " internal":""
 									,belle_sip_request_get_method(belle_sip_transaction_get_request(t))
 									,t);
 		BELLE_SIP_OBJECT_VPTR(t,belle_sip_transaction_t)->on_terminate(t);
@@ -141,10 +146,13 @@ void belle_sip_transaction_notify_timeout(belle_sip_transaction_t *t){
 	 * Otherwise it will report the error.
 	 * We limit this dead channel reporting to REGISTER transactions, who are unlikely to be unresponded.
 	**/
-	belle_sip_warning("Transaction [%p] reporting timeout, reporting to channel.",t);
 	
-	if (strcmp(belle_sip_request_get_method(t->request),"REGISTER")==0 && belle_sip_channel_notify_timeout(t->channel)==TRUE){
-		t->timed_out=TRUE;
+
+	if (strcmp(belle_sip_request_get_method(t->request),"REGISTER")==0){
+		if ( belle_sip_channel_notify_timeout(t->channel)==TRUE){
+			belle_sip_warning("Transaction [%p] reporting timeout, reporting to channel.",t);
+			t->timed_out=TRUE;
+		}
 	}else {
 		notify_timeout(t);
 		belle_sip_transaction_terminate(t);
@@ -161,6 +169,7 @@ static void belle_sip_transaction_reset_dialog(belle_sip_transaction_t *tr, bell
 }
 
 void belle_sip_transaction_set_dialog(belle_sip_transaction_t *t, belle_sip_dialog_t *dialog){
+	if (t->dialog==dialog) return;
 	if (dialog) belle_sip_object_weak_ref(dialog,(belle_sip_object_destroy_notify_t)belle_sip_transaction_reset_dialog,t);
 	if (t->dialog) belle_sip_object_weak_unref(t->dialog,(belle_sip_object_destroy_notify_t)belle_sip_transaction_reset_dialog,t);
 	t->dialog=dialog;
@@ -190,11 +199,10 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 void belle_sip_server_transaction_init(belle_sip_server_transaction_t *t, belle_sip_provider_t *prov,belle_sip_request_t *req){
 	const char *branch;
 	belle_sip_header_via_t *via=BELLE_SIP_HEADER_VIA(belle_sip_message_get_header((belle_sip_message_t*)req,"via"));
-	
 	branch=belle_sip_header_via_get_branch(via);
-	if (branch==NULL){
+	if (branch==NULL || strncmp(branch,BELLE_SIP_BRANCH_MAGIC_COOKIE,strlen(BELLE_SIP_BRANCH_MAGIC_COOKIE))!=0){
 		branch=req->rfc2543_branch;
-		if (branch==NULL) belle_sip_fatal("No computed branch for RFC2543 style of message, this should never happen."); 
+		if (branch==NULL) belle_sip_fatal("No computed branch for RFC2543 style of message, this should never happen.");
 	}
 	t->base.branch_id=belle_sip_strdup(branch);
 	belle_sip_transaction_init((belle_sip_transaction_t*)t,prov,req);
@@ -206,7 +214,7 @@ void belle_sip_server_transaction_send_response(belle_sip_server_transaction_t *
 	belle_sip_header_to_t *to=(belle_sip_header_to_t*)belle_sip_message_get_header((belle_sip_message_t*)resp,"to");
 	belle_sip_dialog_t *dialog=base->dialog;
 	int status_code;
-	
+
 	belle_sip_object_ref(resp);
 	if (!base->last_response){
 		belle_sip_hop_t* hop=belle_sip_response_get_return_hop(resp);
@@ -329,8 +337,9 @@ int belle_sip_client_transaction_send_request_to(belle_sip_client_transaction_t
 	belle_sip_channel_t *chan;
 	belle_sip_provider_t *prov=t->base.provider;
 	belle_sip_dialog_t *dialog=t->base.dialog;
+	belle_sip_request_t *req=t->base.request;
 	int result=-1;
-	
+
 	if (t->base.state!=BELLE_SIP_TRANSACTION_INIT){
 		belle_sip_error("belle_sip_client_transaction_send_request: bad state.");
 		return -1;
@@ -346,26 +355,28 @@ int belle_sip_client_transaction_send_request_to(belle_sip_client_transaction_t
 		t->preset_route=outbound_proxy;
 		belle_sip_object_ref(t->preset_route);
 	}
-	
-	if (t->base.request->dialog_queued){
+
+	if (t->base.sent_by_dialog_queue){
+		
+		/*it can be sent immediately, so update the request with latest cseq and route_set */
+		/*update route and contact just in case they changed*/
+		belle_sip_dialog_update_request(dialog,req);
+	} else if (t->base.request->dialog_queued){
 		/*this request was created by belle_sip_dialog_create_queued_request().*/
-		if (belle_sip_dialog_request_pending(dialog)){
+		if (belle_sip_dialog_request_pending(dialog) || dialog->queued_ct!=NULL){
 			/*it cannot be sent immediately, queue the transaction into dialog*/
-			belle_sip_message("belle_sip_client_transaction_send_request(): transaction [%p], cannot send request now because dialog is busy, so queuing into dialog.",t);
+			belle_sip_message("belle_sip_client_transaction_send_request(): transaction [%p], cannot send request now because dialog is busy"
+			" or other transactions are queued, so queuing into dialog.",t);
 			belle_sip_dialog_queue_client_transaction(dialog,t);
 			return 0;
-		}else{
-			belle_sip_request_t *req=t->base.request;
-			/*it can be sent immediately, so update the request with latest cseq and route_set */
-			/*update route and contact just in case they changed*/
-			belle_sip_dialog_update_request(dialog,req);
 		}
+		belle_sip_dialog_update_request(dialog,req);
 	}
-	
+
 	if (dialog){
 		belle_sip_dialog_update(dialog,(belle_sip_transaction_t*)t,BELLE_SIP_OBJECT_IS_INSTANCE_OF(t,belle_sip_server_transaction_t));
 	}
-	
+
 	if (!t->next_hop) {
 		if (t->preset_route) {
 			t->next_hop=belle_sip_hop_new_from_uri(t->preset_route);
@@ -403,7 +414,7 @@ static unsigned int should_dialog_be_created(belle_sip_client_transaction_t *t,
 	belle_sip_request_t* req = belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(t));
 	const char* method = belle_sip_request_get_method(req);
 	int status_code = belle_sip_response_get_status_code(resp);
-	return status_code>=180 && status_code<300 && (strcmp(method,"INVITE")==0 || strcmp(method,"SUBSCRIBE")==0);
+	return status_code>=101 && status_code<300 && (strcmp(method,"INVITE")==0 || strcmp(method,"SUBSCRIBE")==0);
 }
 
 void belle_sip_client_transaction_notify_response(belle_sip_client_transaction_t *t, belle_sip_response_t *resp){
@@ -418,15 +429,14 @@ void belle_sip_client_transaction_notify_response(belle_sip_client_transaction_t
 	base->last_response=(belle_sip_response_t*)belle_sip_object_ref(resp);
 
 	if (dialog){
-		if (status_code>=200 && status_code<300
+		if (status_code>=101 && status_code<300
 			&& strcmp(method,"INVITE")==0
 			&& (dialog->state==BELLE_SIP_DIALOG_EARLY || dialog->state==BELLE_SIP_DIALOG_CONFIRMED)){
 			/*make sure this response matches the current dialog, or creates a new one*/
 			if (!belle_sip_dialog_match(dialog,(belle_sip_message_t*)resp,FALSE)){
-				dialog=belle_sip_provider_create_dialog_internal(t->base.provider,BELLE_SIP_TRANSACTION(t),FALSE);/*belle_sip_dialog_new(base);*/
-				if (dialog){
-					/*copy userdata to avoid application from being lost*/
-					belle_sip_dialog_set_application_data(dialog,belle_sip_dialog_get_application_data(base->dialog));
+				dialog=belle_sip_provider_find_dialog_from_message(t->base.provider,(belle_sip_message_t*)resp,FALSE);
+				if (!dialog){
+					dialog=belle_sip_provider_create_dialog_internal(t->base.provider,BELLE_SIP_TRANSACTION(t),FALSE);/*belle_sip_dialog_new(base);*/
 					belle_sip_message("Handling response creating a new dialog !");
 				}
 			}
@@ -469,7 +479,7 @@ static void on_channel_state_changed(belle_sip_channel_listener_t *l, belle_sip_
 	belle_sip_client_transaction_t *t=(belle_sip_client_transaction_t*)l;
 	belle_sip_io_error_event_t ev;
 	belle_sip_transaction_state_t tr_state=belle_sip_transaction_get_state((belle_sip_transaction_t*)t);
-	
+
 	belle_sip_message("transaction [%p] channel state changed to [%s]"
 						,t
 						,belle_sip_channel_state_to_string(state));
@@ -493,7 +503,7 @@ static void on_channel_state_changed(belle_sip_channel_listener_t *l, belle_sip_
 			}
 			if (t->base.timed_out)
 				notify_timeout((belle_sip_transaction_t*)t);
-			
+
 			belle_sip_transaction_terminate(BELLE_SIP_TRANSACTION(t));
 		break;
 		default:
@@ -530,7 +540,7 @@ void belle_sip_client_transaction_init(belle_sip_client_transaction_t *obj, bell
 	if (!via){
 		belle_sip_fatal("belle_sip_client_transaction_init(): No via in request.");
 	}
-	
+
 	if (strcmp(belle_sip_request_get_method(req),"CANCEL")!=0){
 		obj->base.branch_id=belle_sip_strdup_printf(BELLE_SIP_BRANCH_MAGIC_COOKIE ".%s",belle_sip_random_token(token,sizeof(token)));
 		belle_sip_header_via_set_branch(via,obj->base.branch_id);
@@ -544,7 +554,7 @@ belle_sip_refresher_t* belle_sip_client_transaction_create_refresher(belle_sip_c
 	return belle_sip_refresher_new(t);
 }
 
-belle_sip_request_t* belle_sip_client_transaction_create_authenticated_request(belle_sip_client_transaction_t *t,belle_sip_list_t** auth_infos) {
+belle_sip_request_t* belle_sip_client_transaction_create_authenticated_request(belle_sip_client_transaction_t *t,belle_sip_list_t** auth_infos,const char* realm) {
 	belle_sip_request_t* initial_request=belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(t));
 	belle_sip_request_t* req=belle_sip_request_clone_with_body(initial_request);
 	belle_sip_header_cseq_t* cseq=belle_sip_message_get_header_by_type(req,belle_sip_header_cseq_t);
@@ -562,7 +572,7 @@ belle_sip_request_t* belle_sip_client_transaction_create_authenticated_request(b
 	belle_sip_message_remove_header(BELLE_SIP_MESSAGE(req),BELLE_SIP_PROXY_AUTHORIZATION);
 
 	/*put auth header*/
-	belle_sip_provider_add_authorization(t->base.provider,req,t->base.last_response,auth_infos);
+	belle_sip_provider_add_authorization(t->base.provider,req,t->base.last_response,NULL,auth_infos,realm);
 	return req;
 }
 
