@@ -76,13 +76,13 @@ static void test_complex_contact_header(void) {
 
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_contact));
 
-	L_contact = belle_sip_header_contact_parse("Contact: toto <sip:titi.com>;expires=3600; q=0.7");
+	L_contact = belle_sip_header_contact_parse("Contact: super toto <sip:titi.com>;expires=3600; q=0.7");
 	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_contact));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_contact));
 	L_contact = belle_sip_header_contact_parse(l_raw_header);
 	belle_sip_free(l_raw_header);
 
-	CU_ASSERT_STRING_EQUAL(belle_sip_header_address_get_displayname((belle_sip_header_address_t*)L_contact), "toto");
+	CU_ASSERT_STRING_EQUAL(belle_sip_header_address_get_displayname((belle_sip_header_address_t*)L_contact), "super toto");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_contact));
 	CU_ASSERT_PTR_NULL(belle_sip_header_contact_parse("m:sip:titi.com, nimportequoi"));
 }
@@ -90,6 +90,7 @@ static void test_complex_contact_header(void) {
 static void test_from_header(void) {
 	belle_sip_header_from_t* L_tmp;
 	belle_sip_uri_t* L_uri;
+	belle_generic_uri_t* L_absoluteUri;
 	belle_sip_header_from_t* L_from = belle_sip_header_from_parse("f:<sip:titi.com;transport=tcp>;tag=dlfjklcn6545614XX");
 	char* l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_from));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_from));
@@ -105,6 +106,22 @@ static void test_from_header(void) {
 	CU_ASSERT_STRING_EQUAL(belle_sip_header_from_get_tag(L_from),"dlfjklcn6545614XX");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_from));
 
+	L_from = belle_sip_header_from_parse("From: <tel:+33124585454;toto=titi>;tag=dlfjklcn6545614XX");
+	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_from));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_from));
+	L_tmp = belle_sip_header_from_parse(l_raw_header);
+	L_from = BELLE_SIP_HEADER_FROM(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
+	belle_sip_free(l_raw_header);
+
+	L_absoluteUri = belle_sip_header_address_get_absolute_uri(BELLE_SIP_HEADER_ADDRESS(L_from));
+	CU_ASSERT_PTR_NULL(belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_from)));
+	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_opaque_part(L_absoluteUri), "+33124585454;toto=titi");
+	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(L_absoluteUri), "tel");
+	CU_ASSERT_STRING_EQUAL(belle_sip_header_from_get_tag(L_from),"dlfjklcn6545614XX");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_from));
+
+
 	/*test factory*/
 	L_from = belle_sip_header_from_create2("super <sip:titi.com:5060;lr;ttl=1;method=INVITE;maddr=192.168.0.1;transport=tcp?header=123>","12345-abc");
 	CU_ASSERT_PTR_NOT_NULL_FATAL(L_from);
@@ -126,17 +143,48 @@ static void test_from_header(void) {
 }
 
 static void test_from_header_with_paramless_address_spec(void) {
+	belle_generic_uri_t *L_absoluteUri;
 	belle_sip_header_from_t* L_from = belle_sip_header_from_parse("From: sip:bob@titi.com;tag=dlfjklcn6545614XX");
 	CU_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_header_from_get_tag(L_from));
 	CU_ASSERT_STRING_EQUAL(belle_sip_header_from_get_tag(L_from),"dlfjklcn6545614XX");
 	belle_sip_object_unref(L_from);
+
+	L_from = belle_sip_header_from_parse("From: tel:1234567;tag=dlfjklcn6545614XX");
+	CU_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_header_from_get_tag(L_from));
+	L_absoluteUri = belle_sip_header_address_get_absolute_uri(BELLE_SIP_HEADER_ADDRESS(L_from));
+	CU_ASSERT_PTR_NULL(belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_from)));
+	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_opaque_part(L_absoluteUri), "1234567");
+	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(L_absoluteUri), "tel");
+	CU_ASSERT_STRING_EQUAL(belle_sip_header_from_get_tag(L_from),"dlfjklcn6545614XX");
+	belle_sip_object_unref(L_from);
+
 }
 
 static void test_to_header_with_paramless_address_spec(void) {
+	belle_generic_uri_t *L_absoluteUri;
 	belle_sip_header_to_t* L_to = belle_sip_header_to_parse("To: sip:bob@titi.com;tag=dlfjklcn6545614XX");
+	belle_sip_uri_t *L_uri;
 	CU_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_header_to_get_tag(L_to));
 	CU_ASSERT_STRING_EQUAL(belle_sip_header_to_get_tag(L_to),"dlfjklcn6545614XX");
 	belle_sip_object_unref(L_to);
+
+	L_to = belle_sip_header_to_parse("To:sip:1002@192.168.1.199;tag=as1f0a0817");
+	CU_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_header_to_get_tag(L_to));
+	L_uri = belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_to));
+	CU_ASSERT_STRING_EQUAL(belle_sip_uri_get_user(L_uri),"1002");
+	CU_ASSERT_STRING_EQUAL(belle_sip_uri_get_host(L_uri), "192.168.1.199");
+
+	CU_ASSERT_STRING_EQUAL(belle_sip_header_to_get_tag(L_to),"as1f0a0817");
+	belle_sip_object_unref(L_to);
+
+	L_to = belle_sip_header_to_parse("To: tel:1234567;tag=dlfjklcn6545614XX");
+	CU_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_header_to_get_tag(L_to));
+	L_absoluteUri = belle_sip_header_address_get_absolute_uri(BELLE_SIP_HEADER_ADDRESS(L_to));
+	CU_ASSERT_PTR_NULL(belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_to)));
+	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_opaque_part(L_absoluteUri), "1234567");
+	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(L_absoluteUri), "tel");
+	CU_ASSERT_STRING_EQUAL(belle_sip_header_to_get_tag(L_to),"dlfjklcn6545614XX");
+	belle_sip_object_unref(L_to);
 	CU_ASSERT_PTR_NULL(belle_sip_header_to_parse("nimportequoi"));
 }
 
@@ -148,7 +196,8 @@ static void test_contact_header_with_paramless_address_spec(void) {
 
 static void test_to_header(void) {
 	belle_sip_uri_t* L_uri;
-	belle_sip_header_to_t* L_to = belle_sip_header_to_parse("t : < sip:titi.com;transport=tcp> ; tag = dlfjklcn6545614XX");
+	belle_generic_uri_t* L_absoluteUri;
+	belle_sip_header_to_t *L_tmp, *L_to = belle_sip_header_to_parse("t : < sip:titi.com;transport=tcp> ; tag = dlfjklcn6545614XX");
 	char* l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_to));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_to));
 	L_to = belle_sip_header_to_parse(l_raw_header);
@@ -177,6 +226,22 @@ static void test_to_header(void) {
 	CU_ASSERT_STRING_EQUAL(belle_sip_header_to_get_tag(L_to),"12345-abc");
 	belle_sip_object_unref(L_to);
 
+	L_to = belle_sip_header_to_parse("To: <tel:+33124585454;toto=titi>;tag=dlfjklcn6545614XX");
+	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_to));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_to));
+	L_tmp = belle_sip_header_to_parse(l_raw_header);
+	L_to = BELLE_SIP_HEADER_TO(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
+	belle_sip_free(l_raw_header);
+
+	L_absoluteUri = belle_sip_header_address_get_absolute_uri(BELLE_SIP_HEADER_ADDRESS(L_to));
+	CU_ASSERT_PTR_NULL(belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_to)));
+	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_opaque_part(L_absoluteUri), "+33124585454;toto=titi");
+	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(L_absoluteUri), "tel");
+	CU_ASSERT_STRING_EQUAL(belle_sip_header_to_get_tag(L_to),"dlfjklcn6545614XX");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_to));
+
+
 }
 
 static void test_via_header(void) {
@@ -390,10 +455,10 @@ static void test_content_length_header(void) {
 
 static belle_sip_header_t* test_header_extension(const char* name,const char* value) {
 	belle_sip_header_t* L_tmp;
+	belle_sip_header_t* L_extension;
 	char header[256];
 	char* l_raw_header=NULL;
 	snprintf(header,sizeof(header),"%s:%s",name,value);
-	belle_sip_header_t* L_extension;
 	L_extension = belle_sip_header_parse(header);
 	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_extension));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_extension));
@@ -489,10 +554,14 @@ static void check_header_authenticate(belle_sip_header_www_authenticate_t* authe
 
 static void test_www_authenticate_header(void) {
 	const char* l_header = "WWW-Authenticate: Digest "
-			"algorithm=MD5, realm=\"atlanta.com\", opaque=\"1bc7f9097684320\","
+			"algorithm=MD5,\r\n   realm=\"atlanta.com\",\r\n  opaque=\"1bc7f9097684320\","
 			" qop=\"auth,auth-int\", nonce=\"c60f3082ee1212b402a21831ae\", stale=true, domain=\"sip:boxesbybob.com\"";
+	const char* l_header_1 = "WWW-Authenticate: Digest realm=\"toto.com\",\r\n   nonce=\"b543409177c9036dbf3054aea940e9703dc8f84c0108\""
+			",\r\n   opaque=\"ALU:QbkRBthOEgEQAkgVEwwHRAIBHgkdHwQCQ1lFRkRWEAkic203MSEzJCgoZS8iI3VlYWRj\",\r\n   algorithm=MD5"
+			",\r\n   qop=\"auth\"";
+
 	belle_sip_header_www_authenticate_t* L_tmp;
-	belle_sip_header_www_authenticate_t* l_authenticate = belle_sip_header_www_authenticate_parse(l_header);
+	belle_sip_header_www_authenticate_t *l_authenticate = belle_sip_header_www_authenticate_parse(l_header);
 
 
 	char* l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(l_authenticate));
@@ -502,6 +571,15 @@ static void test_www_authenticate_header(void) {
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
 	belle_sip_free(l_raw_header);
 	check_header_authenticate(l_authenticate);
+
+
+	l_authenticate = belle_sip_header_www_authenticate_parse(l_header_1);
+	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(l_authenticate));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(l_authenticate));
+	L_tmp = belle_sip_header_www_authenticate_parse(l_raw_header);
+	l_authenticate = BELLE_SIP_HEADER_WWW_AUTHENTICATE(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(l_authenticate));
+
 	CU_ASSERT_PTR_NULL(belle_sip_header_www_authenticate_parse("nimportequoi"));
 
 }
@@ -616,8 +694,15 @@ static void test_address_with_error_header(void) {
 
 static void test_address_header(void) {
 	belle_sip_uri_t* L_uri;
+	char* L_raw;
 	belle_sip_header_address_t* laddress = belle_sip_header_address_parse("\"toto\" <sip:liblinphone_tester@81.56.11.2:5060>");
-	CU_ASSERT_PTR_NOT_NULL_FATAL(laddress);
+	CU_ASSERT_PTR_NOT_NULL(laddress);
+	L_raw = belle_sip_object_to_string(BELLE_SIP_OBJECT(laddress));
+	CU_ASSERT_PTR_NOT_NULL(L_raw);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
+	laddress = belle_sip_header_address_parse(L_raw);
+	belle_sip_free(L_raw);
+
 	CU_ASSERT_STRING_EQUAL("toto",belle_sip_header_address_get_displayname(laddress))
 	L_uri = belle_sip_header_address_get_uri(laddress);
 
@@ -626,6 +711,38 @@ static void test_address_header(void) {
 	CU_ASSERT_STRING_EQUAL(belle_sip_uri_get_user(L_uri), "liblinphone_tester");
 	CU_ASSERT_EQUAL(belle_sip_uri_get_port(L_uri), 5060);
 	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
+
+	laddress = belle_sip_header_address_parse("\"\\\"to\\\\to\\\"\" <sip:liblinphone_tester@81.56.11.2:5060>");
+	CU_ASSERT_PTR_NOT_NULL(laddress);
+	L_raw = belle_sip_object_to_string(BELLE_SIP_OBJECT(laddress));
+	CU_ASSERT_PTR_NOT_NULL(L_raw);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
+	laddress = belle_sip_header_address_parse(L_raw);
+	belle_sip_free(L_raw);
+	CU_ASSERT_STRING_EQUAL("\"to\\to\"",belle_sip_header_address_get_displayname(laddress))
+	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
+
+
+
+}
+
+static void test_address_header_with_tel_uri(void) {
+	belle_generic_uri_t* L_uri;
+	char* L_raw;
+	belle_sip_header_address_t* laddress = belle_sip_header_address_parse("\"toto\" <tel:123456>");
+	CU_ASSERT_PTR_NOT_NULL(laddress);
+	L_raw = belle_sip_object_to_string(BELLE_SIP_OBJECT(laddress));
+	CU_ASSERT_PTR_NOT_NULL(L_raw);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
+	laddress = belle_sip_header_address_parse(L_raw);
+	belle_sip_free(L_raw);
+
+	CU_ASSERT_STRING_EQUAL("toto",belle_sip_header_address_get_displayname(laddress));
+	L_uri = belle_sip_header_address_get_absolute_uri(laddress);
+
+	CU_ASSERT_PTR_NOT_NULL(belle_generic_uri_get_opaque_part(L_uri));
+	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_opaque_part(L_uri), "123456");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
 }
 
 static void test_very_long_address_header(void) {
@@ -749,20 +866,47 @@ static void test_replaces_escaped_header(void) {
 
 }
 
-static void test_date_header(void){
+#ifndef WIN32
+static void _test_date_header(void){
 	belle_sip_header_date_t *date,*date2;
 	time_t utc;
 #define DATE_EXAMPLE "Thu, 21 Feb 2002 13:02:03 GMT"
+#define DATE_UTC_EXAMPLE 1014296523L /* the above date in UTC */
 	date=belle_sip_header_date_parse("Date: " DATE_EXAMPLE);
 	CU_ASSERT_PTR_NOT_NULL(date);
 	utc=belle_sip_header_date_get_time(date);
-	CU_ASSERT_TRUE(utc!=(time_t)-1);
-	
+	CU_ASSERT_EQUAL(utc,DATE_UTC_EXAMPLE);
+
 	date2=belle_sip_header_date_create_from_time(&utc);
 	CU_ASSERT_PTR_NOT_NULL(date2);
 	CU_ASSERT_TRUE(strcmp(belle_sip_header_date_get_date(date2),DATE_EXAMPLE)==0);
 	CU_ASSERT_PTR_NULL(belle_sip_header_date_parse("nimportequoi"));
+}
 
+#endif
+
+static void test_date_header(void){
+#ifdef WIN32
+	// TODO: setenv and unsetenv are not available for Windows
+#else
+	char *tz;
+	/* test in our timezone */
+	_test_date_header();
+
+	/* test within another timezone */
+	tz = getenv("TZ");
+	setenv("TZ","Etc/GMT+4",1);
+	tzset();
+
+	_test_date_header();
+
+	/* reset to original timezone */
+	if(tz)
+		setenv("TZ", tz, 1);
+	else
+		unsetenv("TZ");
+	tzset();
+#endif
 }
 
 static void test_p_preferred_identity_header(void) {
@@ -814,8 +958,25 @@ static void test_privacy_header() {
 	test_privacy("Privacy: id",value2,1);
 }
 
+static void test_event_header(void) {
+	belle_sip_header_event_t* L_tmp;
+	belle_sip_header_event_t* L_event = belle_sip_header_event_parse("Event: presence;id=blabla1");
+	char* l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_event));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_event));
+	L_tmp = belle_sip_header_event_parse(l_raw_header);
+	L_event = BELLE_SIP_HEADER_EVENT(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
+	belle_sip_free(l_raw_header);
+
+	CU_ASSERT_STRING_EQUAL(belle_sip_header_event_get_package_name(L_event), "presence");
+	CU_ASSERT_STRING_EQUAL(belle_sip_header_event_get_id(L_event), "blabla1");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_event));
+	CU_ASSERT_PTR_NULL(belle_sip_header_event_parse("nimportequoi"));
+}
+
 test_t headers_tests[] = {
 	{ "Address", test_address_header },
+	{ "Address tel uri", test_address_header_with_tel_uri },
 	{ "Header address (very long)", test_very_long_address_header },
 	{ "Address with error", test_address_with_error_header },
 	{ "Allow", test_allow_header },
@@ -850,7 +1011,8 @@ test_t headers_tests[] = {
 	{ "Via", test_via_header },
 	{ "WWW-Authenticate", test_www_authenticate_header },
 	{ "Header extension", test_header_extension_1 },
-	{ "Header extension 2", test_header_extension_2 }
+	{ "Header extension 2", test_header_extension_2 },
+	{ "Header event", test_event_header }
 
 };
 
