@@ -227,7 +227,7 @@ static void server_process_request_event(void *obj, const belle_sip_request_even
 
 			belle_sip_header_t* sip_if_match=belle_sip_message_get_header(BELLE_SIP_MESSAGE(resp),"SIP-If-Match");
 			if (sip_if_match) {
-				CU_ASSERT_EQUAL(belle_sip_header_extension_get_value(BELLE_SIP_HEADER_EXTENSION(sip_if_match)),"blablietag");
+				CU_ASSERT_STRING_EQUAL(belle_sip_header_extension_get_value(BELLE_SIP_HEADER_EXTENSION(sip_if_match)),"blablietag");
 			}
 			/*check for body*/
 			CU_ASSERT_PTR_NOT_NULL(belle_sip_message_get_body(BELLE_SIP_MESSAGE(req)));
@@ -320,9 +320,9 @@ static endpoint_t* create_endpoint(const char *ip, int port,const char* transpor
 	endpoint->listener_callbacks=listener_callbacks;
 	endpoint->lp=belle_sip_stack_create_listening_point(endpoint->stack,ip,port,transport);
 	endpoint->connection_family=AF_INET;
-	
+
 	if (endpoint->lp) belle_sip_object_ref(endpoint->lp);
-	
+
 	endpoint->provider=belle_sip_stack_create_provider(endpoint->stack,endpoint->lp);
 	belle_sip_provider_add_sip_listener(endpoint->provider,(endpoint->listener=belle_sip_listener_create_from_callbacks(endpoint->listener_callbacks,endpoint)));
 	sprintf(endpoint->nonce,"%p",endpoint); /*initial nonce*/
@@ -363,7 +363,7 @@ static void refresher_base_with_body(endpoint_t* client
 	uint64_t end;
 	if (client->expire_in_contact) belle_sip_header_contact_set_expires(contact,1);
 
-	
+
 	dest_uri=(belle_sip_uri_t*)belle_sip_object_clone((belle_sip_object_t*)belle_sip_listening_point_get_uri(server->lp));
 	if (client->connection_family==AF_INET6)
 		belle_sip_uri_set_host(dest_uri,"::1");
@@ -403,7 +403,7 @@ static void refresher_base_with_body(endpoint_t* client
 		} else {
 			CU_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.fourHundredOne,1,1000));
 			/*update cseq*/
-			req=belle_sip_client_transaction_create_authenticated_request(trans,NULL);
+			req=belle_sip_client_transaction_create_authenticated_request(trans,NULL,NULL);
 			belle_sip_object_unref(trans);
 			trans=belle_sip_provider_create_client_transaction(client->provider,req);
 			belle_sip_object_ref(trans);
@@ -419,12 +419,13 @@ static void refresher_base_with_body(endpoint_t* client
 	begin = belle_sip_time_ms();
 	CU_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.refreshOk,client->register_count+(client->early_refresher?1:0),client->register_count*1000 + 1000));
 	end = belle_sip_time_ms();
-	CU_ASSERT_TRUE(end-begin>=client->register_count*1000);
+	CU_ASSERT_TRUE(end-begin>=client->register_count*1000*.9); /*because refresh is at 90% of expire*/
 	CU_ASSERT_TRUE(end-begin<(client->register_count*1000 + 2000));
 	/*unregister twice to make sure refresh operation can be safely cascaded*/
 	belle_sip_refresher_refresh(refresher,0);
 	belle_sip_refresher_refresh(refresher,0);
 	CU_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.refreshOk,client->register_count+1,1000));
+	CU_ASSERT_EQUAL(client->stat.refreshOk,client->register_count+1);
 	belle_sip_refresher_stop(refresher);
 	belle_sip_object_unref(refresher);
 }
@@ -489,7 +490,7 @@ static void subscribe_test(void) {
 	server = create_udp_endpoint(6788,&server_callbacks);
 	server->expire_in_contact=0;
 	server->auth=digest_auth;
-	
+
 	dest_uri=(belle_sip_uri_t*)belle_sip_object_clone((belle_sip_object_t*)belle_sip_listening_point_get_uri(server->lp));
 	belle_sip_uri_set_host(dest_uri,"127.0.0.1");
 	destination_route=belle_sip_header_route_create(belle_sip_header_address_create(NULL,dest_uri));
@@ -515,7 +516,7 @@ static void subscribe_test(void) {
 
 	CU_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.fourHundredOne,1,1000));
 
-	req=belle_sip_client_transaction_create_authenticated_request(trans,NULL);
+	req=belle_sip_client_transaction_create_authenticated_request(trans,NULL,NULL);
 	belle_sip_object_unref(trans);
 	trans=belle_sip_provider_create_client_transaction(client->provider,req);
 	belle_sip_object_ref(trans);
@@ -628,7 +629,34 @@ static int register_test_with_interfaces(const char *transport, const char *clie
 	client = create_endpoint(client_ip,3452,transport,&client_callbacks);
 	client->connection_family=connection_family;
 	client->register_count=1;
-	
+
+	server = create_endpoint(server_ip,6788,transport,&server_callbacks);
+	server->expire_in_contact=client->expire_in_contact=0;
+	server->auth=none;
+
+	if (client->lp==NULL || server->lp==NULL){
+		belle_sip_warning("Cannot check ipv6 because host has no ipv6 support.");
+		ret=-1;
+	}else register_base(client,server);
+	destroy_endpoint(client);
+	destroy_endpoint(server);
+	return ret;
+}
+
+static int register_test_with_random_port(const char *transport, const char *client_ip, const char *server_ip, int connection_family) {
+	int ret=0;
+	belle_sip_listener_callbacks_t client_callbacks;
+	belle_sip_listener_callbacks_t server_callbacks;
+	endpoint_t* client,*server;
+	memset(&client_callbacks,0,sizeof(belle_sip_listener_callbacks_t));
+	memset(&server_callbacks,0,sizeof(belle_sip_listener_callbacks_t));
+	client_callbacks.process_response_event=client_process_response_event;
+	client_callbacks.process_auth_requested=client_process_auth_requested;
+	server_callbacks.process_request_event=server_process_request_event;
+	client = create_endpoint(client_ip,-1,transport,&client_callbacks);
+	client->connection_family=connection_family;
+	client->register_count=1;
+
 	server = create_endpoint(server_ip,6788,transport,&server_callbacks);
 	server->expire_in_contact=client->expire_in_contact=0;
 	server->auth=none;
@@ -706,6 +734,30 @@ static void register_tcp_test_ipv6_to_ipv6_with_ipv6(void){
 	register_test_with_interfaces("tcp","::0","::0",AF_INET6);
 }
 
+static void register_udp_test_ipv4_random_port(void){
+	register_test_with_random_port("udp","0.0.0.0","0.0.0.0",AF_INET);
+}
+
+static void register_udp_test_ipv6_random_port(void){
+	if (!belle_sip_tester_ipv6_available()){
+		belle_sip_warning("Test skipped, IPv6 connectivity not available.");
+		return;
+	}
+	register_test_with_random_port("udp","::0","0.0.0.0",AF_INET);
+}
+
+static void register_tcp_test_ipv4_random_port(void){
+	register_test_with_random_port("tcp","0.0.0.0","0.0.0.0",AF_INET);
+}
+
+static void register_tcp_test_ipv6_random_port(void){
+	if (!belle_sip_tester_ipv6_available()){
+		belle_sip_warning("Test skipped, IPv6 connectivity not available.");
+		return;
+	}
+	register_test_with_random_port("tcp","::0","0.0.0.0",AF_INET);
+}
+
 static void simple_publish() {
 	belle_sip_header_content_type_t* content_type=belle_sip_header_content_type_create("application","pidf+xml");
 	refresher_base_with_param_and_body("PUBLISH",FALSE,TRUE,FALSE, content_type,publish_body);
@@ -735,7 +787,11 @@ test_t refresher_tests[] = {
 	{ "REGISTER TCP from ipv6 to ipv4", register_tcp_test_ipv6_to_ipv4 },
 	{ "REGISTER TCP from ipv4 to ipv6", register_tcp_test_ipv4_to_ipv6 },
 	{ "REGISTER TCP from ipv6 to ipv6 with ipv4", register_tcp_test_ipv6_to_ipv6_with_ipv4 },
-	{ "REGISTER TCP from ipv6 to ipv6 with ipv6", register_tcp_test_ipv6_to_ipv6_with_ipv6 }
+	{ "REGISTER TCP from ipv6 to ipv6 with ipv6", register_tcp_test_ipv6_to_ipv6_with_ipv6 },
+	{ "REGISTER UDP from random port using AF_INET", register_udp_test_ipv4_random_port },
+	{ "REGISTER UDP from random port using AF_INET6", register_udp_test_ipv6_random_port },
+	{ "REGISTER TCP from random port using AF_INET", register_tcp_test_ipv4_random_port },
+	{ "REGISTER TCP from random port using AF_INET6", register_tcp_test_ipv6_random_port },
 };
 
 test_suite_t refresher_test_suite = {
